import numpy as np
from scipy import signal

def conv2(s1, sig, axes=None):
    if axes is None:
        axes = np.arange(0,len(sig))
    
    axes = np.array(axes)
    if isinstance(sig, (int, float)) or len(sig)==1:
        sig = sig*np.ones((len(axes),1),np.float32)
    elif len(sig)>1 and len(sig) != len(axes):
        raise ValueError('number of axes different from number of smoothing constants')
    
    s1 = np.array(s1).astype(np.float32)
    sdim = s1.ndim
    sig = np.array(sig).astype(np.float32)
    
    sfilt = s1
    for i in range(0,len(axes)):
        dims = np.arange(-1,sdim)
        dims[0] = axes[i]
        dims = np.delete(dims, [axes[i]+1])
        sfilt = np.transpose(sfilt, dims)
        print(dims)
        ns = sfilt.shape[0]
        
        tmax = np.ceil(4*sig[i])
        dt = np.arange(-tmax,tmax)
        gaus = np.exp(-dt**2 / (2*sig[i]**2))
        gaus /= gaus.sum()
        
        flat = np.ones((ns,),np.float32)
        snorm = signal.convolve(flat, gaus, mode='same')
        for j in range(0,sdim-1):
            gaus = np.expand_dims(gaus,axis=-1)
            snorm = np.expand_dims(snorm,axis=-1)
        print(gaus.shape)
        print(snorm.shape)
        t0 = time.time()
        sfilt = signal.convolve(sfilt, gaus, mode='same')
        print(time.time()-t0)
        
        #if sfilt.shape[0] > ns:
        #    icent = np.floor(sfilt.shape[0]/2) - np.floor(ns/2)
        #    inds  = (icent + np.arange(0,ns)).astype(np.int32)
        #    sfilt = sfilt[inds,:]
        #    snorm = snorm[inds]
        #sfilt = sfilt / snorm
        print(sfilt.shape)
        dims = np.arange(1,sdim)
        dims = np.insert(dims, axes[i], 0)
        sfilt = np.transpose(sfilt, dims)
        print(sfilt.shape)
    return sfilt

def conv_circ(s1, sig, axes=None):
    if axes is None:
        axes = np.arange(0,1)
        
    return s1
    


import numpy as np
import multiprocessing
from scipy.ndimage import filters
from multiprocessing import Pool

def oasis1t(inputs):
    F, ops = inputs
    ca = F
    NT = F.shape[0]

    v = np.zeros((NT,))
    w = np.zeros((NT,))
    t = np.zeros((NT,), dtype=int)
    l = np.zeros((NT,))
    s = np.zeros((NT,))

    g = -1./(ops['tau'] * ops['fs'])

    for i in range(0,10):
        it = 0
        ip = 0

        while it<NT:
            v[ip], w[ip],t[ip],l[ip] = ca[it],1,it,1

            while ip>0:
                if v[ip-1] * np.exp(g * l[ip-1]) > v[ip]:
                    # violation of the constraint means merging pools
                    f1 = np.exp(g * l[ip-1])
                    f2 = np.exp(2 * g * l[ip-1])
                    wnew = w[ip-1] + w[ip] * f2
                    v[ip-1] = (v[ip-1] * w[ip-1] + v[ip] * w[ip]* f1) / wnew
                    w[ip-1] = wnew
                    l[ip-1] += l[ip]

                    ip += -1
                else:
                    break
            it += 1
            ip += 1

        s[t[1:ip]] = v[1:ip] - v[:ip-1] * np.exp(g * l[:ip-1])

        return s

def oasis(F, ops):
    num_cores = multiprocessing.cpu_count()
    F = preprocess(F,ops)
    inputs = range(F.shape[0])
    Fsplit = []
    for i in inputs:
        Fsplit.append((F[i,:], ops))
    with Pool(num_cores) as p:
        results = p.map(oasis1t, Fsplit)
    #results = Parallel(n_jobs=num_cores)(delayed(oasis1t)(F[i, :], ops) for i in inputs)
    # collect results as numpy array
    sp = np.zeros_like(F)
    for i in inputs:
        sp[i,:] = results[i]
    return sp


def preprocess(F,ops):
    sig = ops['sig_baseline']
    win = int(ops['win_baseline']*ops['fs'])    
    if ops['baseline']=='maximin':
        Flow = filters.gaussian_filter(F,    [0., sig])
        Flow = filters.minimum_filter1d(Flow,    win)
        Flow = filters.maximum_filter1d(Flow,    win)
    elif ops['baseline']=='constant':
        Flow = filters.gaussian_filter(F,    [0., sig])
        Flow = np.amin(Flow)
    elif ops['baseline']=='constant_prctile':
        Flow = np.percentile(F, ops['prctile_baseline'], axis=1)
        Flow = np.expand_dims(Flow, axis = 1)
    else:
        Flow = 0.

    F = F - Flow

    return F



from PyQt5 import QtGui, QtCore
import numpy as np
import pyqtgraph as pg
from scipy.ndimage import gaussian_filter1d
from scipy.sparse.linalg import eigsh
from scipy.stats import zscore
from matplotlib.colors import hsv_to_rgb
from matplotlib import cm
import time
import sys
from rastermap import mapping
from suite2p import gui,fig


### custom QDialog which allows user to fill in ops and run suite2p!
class VisWindow(QtGui.QMainWindow):
    def __init__(self, parent=None):
        super(VisWindow, self).__init__(parent)
        pg.setConfigOptions(imageAxisOrder='row-major')
        self.setGeometry(70,70,1100,900)
        self.setWindowTitle('Visualize deconvolved data')
        self.cwidget = QtGui.QWidget(self)
        self.setCentralWidget(self.cwidget)
        self.l0 = QtGui.QGridLayout()
        #layout = QtGui.QFormLayout()
        self.cwidget.setLayout(self.l0)
        #self.p0 = pg.ViewBox(lockAspect=False,name='plot1',border=[100,100,100],invertY=True)
        self.win = pg.GraphicsLayoutWidget()
        # --- cells image
        self.win = pg.GraphicsLayoutWidget()
        self.win.move(600,0)
        self.win.resize(1000,500)
        self.l0.addWidget(self.win,0,0,14,14)
        layout = self.win.ci.layout
        # A plot area (ViewBox + axes) for displaying the image
        self.p0 = self.win.addViewBox(row=0,col=0)
        self.p0.setMouseEnabled(x=False,y=False)
        self.p0.setMenuEnabled(False)
        self.p1 = self.win.addPlot(title="FULL VIEW",row=0,col=1)
        self.p1.setMouseEnabled(x=False,y=False)
        self.img = pg.ImageItem(autoDownsample=True)
        self.p1.addItem(self.img)
        # cells to plot
        if len(parent.imerge)==1:
            icell = parent.iscell[parent.imerge[0]]
            self.cells = np.array((parent.iscell==icell).nonzero()).flatten()
        else:
            self.cells = np.array(parent.imerge).flatten()
        # compute spikes
        sp = parent.Spks[self.cells,:]
        sp = np.squeeze(sp)
        sp = zscore(sp, axis=1)
        sp -= sp.min()
        sp /= sp.max()
        sp = np.maximum(0, np.minimum(1, sp))
        self.sp = np.maximum(0,np.minimum(1,sp))
        # 100 ms bins
        self.bin = int(np.maximum(1, int(parent.ops['fs']/10)))
        # draw axes
        self.p1.setXRange(0,sp.shape[1])
        self.p1.setYRange(0,sp.shape[0])
        self.p1.setLimits(xMin=-10,xMax=sp.shape[1]+10,yMin=-10,yMax=sp.shape[0]+10)
        self.p1.setLabel('left', 'neurons')
        self.p1.setLabel('bottom', 'time')
        # zoom in on a selected image region
        nt = sp.shape[1]
        nn = sp.shape[0]
        self.p2 = self.win.addPlot(title='ZOOM IN',row=1,col=0,colspan=2)
        self.imgROI = pg.ImageItem(autoDownsample=True)
        self.p2.addItem(self.imgROI)
        self.p2.setMouseEnabled(x=False,y=False)
        self.p2.setLabel('left', 'neurons')
        self.p2.setLabel('bottom', 'time')
        # set colormap to viridis
        colormap = cm.get_cmap("viridis")
        colormap._init()
        lut = (colormap._lut * 255).view(np.ndarray)  # Convert matplotlib colormap from 0-1 to 0 -255 for Qt
        lut = lut[0:-3,:]
        # apply the colormap
        self.img.setLookupTable(lut)
        self.imgROI.setLookupTable(lut)
        self.img.setLevels([0,1])
        self.imgROI.setLevels([0,1])
        layout.setColumnStretchFactor(1,3)
        layout.setRowStretchFactor(1,3)
        # add slider for levels
        self.sl = QtGui.QSlider(QtCore.Qt.Vertical)
        self.sl.setMinimum(0)
        self.sl.setMaximum(100)
        self.sl.setValue(100)
        self.sl.setTickPosition(QtGui.QSlider.TicksLeft)
        self.sl.setTickInterval(10)
        self.sl.sliderReleased.connect(self.levelchange)
        self.sat = 1.0
        self.l0.addWidget(self.sl,0,2,5,1)
        qlabel = gui.VerticalLabel(text='saturation')
        #qlabel.setStyleSheet('color: white;')
        self.l0.addWidget(qlabel,1,3,3,1)
        self.isort = np.arange(0,self.cells.size).astype(np.int32)
        # ROI on main plot
        redpen = pg.mkPen(pg.mkColor(255, 0, 0),
                                width=3,
                                style=QtCore.Qt.SolidLine)
        self.ROI = pg.RectROI([nt*.25, -1], [nt*.25, nn+1],
                      maxBounds=QtCore.QRectF(-1.,-1.,nt+1,nn+1),
                      pen=redpen)
        self.ROI.handleSize = 14
        self.ROI.handlePen = redpen
        # Add top and right Handles
        self.ROI.addScaleHandle([1, 0.5], [0., 0.5])
        self.ROI.addScaleHandle([0.5, 0], [0.5, 1])
        self.ROI.sigRegionChangeFinished.connect(self.ROI_position)
        self.p1.addItem(self.ROI)
        self.ROI.setZValue(10)  # make sure ROI is drawn above image
        self.neural_sorting(2)
        # buttons for computations
        self.PCOn = QtGui.QPushButton('compute PCs')
        self.PCOn.clicked.connect(lambda: self.PC_on(True))
        self.l0.addWidget(self.PCOn,0,0,1,2)
        self.mapOn = QtGui.QPushButton('compute raster map')
        self.mapOn.clicked.connect(self.map_on)
        self.l0.addWidget(self.mapOn,1,0,1,2)
        self.comboBox = QtGui.QComboBox(self)
        self.l0.addWidget(self.comboBox,2,0,1,2)
        self.l0.addWidget(QtGui.QLabel(''),3,0,1,1)
        #self.l0.addWidget(QtGui.QLabel(''),4,0,1,1)
        self.selectBtn = QtGui.QPushButton('show selected cells in GUI')
        self.selectBtn.clicked.connect(lambda: self.select_cells(parent))
        self.selectBtn.setEnabled(True)
        self.l0.addWidget(self.selectBtn,4,0,1,2)
        self.l0.addWidget(QtGui.QLabel(''),5,0,1,1)
        self.l0.setRowStretch(6, 1)
        self.win.show()
        self.show()

    def levelchange(self):
        self.sat = float(self.sl.value())/100
        self.img.setLevels([0,self.sat])
        self.imgROI.setLevels([0,self.sat])

    def PC_on(self, plot):
        # edit buttons
        self.PCedit = QtGui.QLineEdit(self)
        self.PCedit.setValidator(QtGui.QIntValidator(1,np.minimum(self.sp.shape[0],self.sp.shape[1])))
        self.PCedit.setText('1')
        self.PCedit.setFixedWidth(60)
        self.PCedit.setAlignment(QtCore.Qt.AlignRight)
        qlabel = QtGui.QLabel('PC: ')
        qlabel.setStyleSheet('color: white;')
        self.l0.addWidget(qlabel,3,0,1,1)
        self.l0.addWidget(self.PCedit,3,1,1,1)
        self.comboBox.addItem("PC")
        self.PCedit.returnPressed.connect(self.PCreturn)
        self.compute_svd(self.bin)
        self.comboBox.currentIndexChanged.connect(self.neural_sorting)
        if plot:
            self.neural_sorting(0)
        self.PCOn.setEnabled(False)

    def PCreturn(self):
        self.comboBox.setCurrentIndex(0)
        self.neural_sorting(0)

    def map_on(self):
        if not hasattr(self,'u'):
            self.PC_on(False)
        self.comboBox.addItem('raster map')
        tic = time.time()
        self.compute_map()
        print('raster map computed in %3.2f s'%(time.time()-tic))
        self.comboBox.setCurrentIndex(1)
        self.comboBox.currentIndexChanged.connect(self.neural_sorting)
        self.neural_sorting(1)
        self.PCOn.setEnabled(False)
        self.mapOn.setEnabled(False)

    def compute_map(self):
        self.isort1, self.isort2 = mapping.main(self.sp,None,self.u,self.sv,self.v)

    def compute_svd(self,bin):
        sp = self.sp[:,:int(np.floor(self.sp.shape[1]/bin)*bin)]
        spbin = sp.reshape((sp.shape[0],bin,int(sp.shape[1]/bin)))
        spbin = np.squeeze(spbin.mean(axis=1))
        tic=time.time()
        nmin = min([spbin.shape[0],spbin.shape[1]])
        nmin = np.minimum(nmin-1, 200)
        spbin -= spbin.mean(axis=1)[:,np.newaxis]
        self.sv,self.u = eigsh(np.dot(spbin,spbin.T), k=nmin)
        # these eigenvectors are in the opposite order
        self.sv = self.sv[::-1]
        self.u  = self.u[:,::-1]
        self.sv = self.sv ** 0.5
        self.v = (self.sp-self.sp.mean(axis=1)[:,np.newaxis]).T @ self.u
        print('svd computed in %3.2f s'%(time.time()-tic))

    def ROI_position(self):
        pos = self.ROI.pos()
        posy = pos.y()
        posx = pos.x()
        sizex,sizey = self.ROI.size()
        xrange = (np.arange(1,int(sizex)) + int(posx)).astype(np.int32)
        yrange = (np.arange(1,int(sizey)) + int(posy)).astype(np.int32)
        xrange = xrange[xrange>=0]
        xrange = xrange[xrange<self.sp.shape[1]]
        yrange = yrange[yrange>=0]
        yrange = yrange[yrange<self.sp.shape[0]]
        self.imgROI.setImage(self.spF[np.ix_(yrange,xrange)])
        #self.selected = (self.sp.shape[0]-1) - yrange
        self.selected = yrange
        self.p2.setXRange(0,xrange.size)
        self.p2.setYRange(0,yrange.size)
        axy = self.p2.getAxis('left')
        axx = self.p2.getAxis('bottom')
        axy.setTicks([[(0.0,str(yrange[0])),(float(yrange.size),str(yrange[-1]))]])
        axx.setTicks([[(0.0,str(xrange[0])),(float(xrange.size),str(xrange[-1]))]])
        # bug in range, going to comment out for now
        #self.imgROI.setRect(QtCore.QRectF(xrange[0],yrange[0],
        #                                   xrange[-1]-xrange[0],yrange[-1]-yrange[0]))
        self.imgROI.setLevels([0,self.sat])

    def select_cells(self,parent):
        parent.imerge = []
        if self.selected.size < 1000:
            for n in self.selected:
                parent.imerge.append(self.cells[self.isort[n]])
            parent.ichosen = parent.imerge[0]
            parent.ichosen_stats()
            M = fig.draw_masks(parent)
            fig.plot_masks(parent,M)
            fig.plot_trace(parent)
            parent.show()
        else:
            print('too many cells selected')


    def neural_sorting(self,i):
        if i==0:
            self.isort = np.argsort(self.u[:,int(self.PCedit.text())-1])
        elif i==1:
            self.isort = self.isort1
        if i<2:
            self.spF = gaussian_filter1d(self.sp[self.isort,:].T,
                                        np.minimum(8,np.maximum(1,int(self.sp.shape[0]*0.005))),
                                        axis=1)
            self.spF = self.spF.T
        else:
            self.spF = self.sp
        self.spF = zscore(self.spF, axis=1)
        self.spF = np.minimum(8, self.spF)
        self.spF = np.maximum(0, self.spF)
        self.spF /= 8
        self.img.setImage(self.spF)
        self.ROI_position()



from PyQt5 import QtGui, QtCore
from suite2p import fig, gui, classifier, visualize
import pyqtgraph as pg
import numpy as np
import sys
import os
import shutil
import time
#class EventWidget(QtGui.QWidget):
#    def __init__(self,parent=None):
#        super(EventWidget, self, parent).__init__()
#    def

class MainW(QtGui.QMainWindow):
    def __init__(self):
        super(MainW, self).__init__()
        pg.setConfigOptions(imageAxisOrder='row-major')
        self.setGeometry(25,25,1600,1000)
        self.setWindowTitle('suite2p (run pipeline or load stat.npy)')
        icon_path = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                         'logo/logo.png')
        app_icon = QtGui.QIcon()
        app_icon.addFile(icon_path, QtCore.QSize(16,16))
        app_icon.addFile(icon_path, QtCore.QSize(24,24))
        app_icon.addFile(icon_path, QtCore.QSize(32,32))
        app_icon.addFile(icon_path, QtCore.QSize(48,48))
        app_icon.addFile(icon_path, QtCore.QSize(96,96))
        app_icon.addFile(icon_path, QtCore.QSize(256,256))
        self.setWindowIcon(app_icon)
        self.setStyleSheet("QMainWindow {background: 'black';}")
        self.stylePressed = "QPushButton {Text-align: left; background-color: rgb(100,50,100); color:white;}"
        self.styleUnpressed = "QPushButton {Text-align: left; background-color: rgb(50,50,50); color:white;}"
        self.styleInactive = "QPushButton {Text-align: left; background-color: rgb(50,50,50); color:gray;}"
        self.loaded = False
        self.ops_plot = []
        # default plot options
        self.ops_plot.append(True)
        self.ops_plot.append(0)
        self.ops_plot.append(0)
        self.ops_plot.append(0)
        self.ops_plot.append(0)
        #### ------ MENU BAR ----------------- ####
        # run suite2p from scratch
        runS2P =  QtGui.QAction('&Run suite2p ', self)
        runS2P.setShortcut('Ctrl+R')
        runS2P.triggered.connect(self.run_suite2p)
        self.addAction(runS2P)
        # load processed data
        loadProc = QtGui.QAction('&Load processed data (choose stat.npy file)', self)
        loadProc.setShortcut('Ctrl+L')
        loadProc.triggered.connect(self.load_dialog)
        self.addAction(loadProc)
        # load masks
        #loadMask = QtGui.QAction('&Load masks (stat.npy) and extract traces', self)
        #loadMask.setShortcut('Ctrl+M')
        #self.addAction(loadMask)
        # make mainmenu!
        main_menu = self.menuBar()
        file_menu = main_menu.addMenu('&File')
        file_menu.addAction(runS2P)
        file_menu.addAction(loadProc)
        # classifier menu
        self.trainfiles = []
        self.statlabels = None
        self.statclass = ['skew','compact','aspect_ratio','footprint']
        self.loadMenu = QtGui.QMenu('Load', self)
        self.loadClass = QtGui.QAction('from file', self)
        self.loadClass.triggered.connect(self.load_classifier)
        self.loadClass.setEnabled(False)
        self.loadMenu.addAction(self.loadClass)
        self.loadUClass = QtGui.QAction('default classifier', self)
        self.loadUClass.triggered.connect(self.load_default_classifier)
        self.loadUClass.setEnabled(False)
        self.loadMenu.addAction(self.loadUClass)
        self.loadSClass = QtGui.QAction('built-in classifier', self)
        self.loadSClass.triggered.connect(self.load_s2p_classifier)
        self.loadSClass.setEnabled(False)
        self.loadMenu.addAction(self.loadSClass)
        self.loadTrain = QtGui.QAction('Build', self)
        self.loadTrain.triggered.connect(lambda: classifier.load_list(self))
        self.loadTrain.setEnabled(False)
        self.saveDefault = QtGui.QAction('Save loaded as default', self)
        self.saveDefault.triggered.connect(self.class_default)
        self.saveDefault.setEnabled(False)
        self.resetDefault =  QtGui.QAction('Reset default to built-in', self)
        self.resetDefault.triggered.connect(self.reset_default)
        self.resetDefault.setEnabled(False)
        class_menu = main_menu.addMenu('&Classifier')
        class_menu.addMenu(self.loadMenu)
        class_menu.addAction(self.loadTrain)
        class_menu.addAction(self.resetDefault)
        class_menu.addAction(self.saveDefault)
        # visualizations menuBar
        self.visualizations = QtGui.QAction('&Visualize selected cells', self)
        self.visualizations.triggered.connect(self.vis_window)
        self.visualizations.setEnabled(False)
        vis_menu = main_menu.addMenu('&Visualizations')
        vis_menu.addAction(self.visualizations)
        self.visualizations.setShortcut('Ctrl+V')
        #### --------- MAIN WIDGET LAYOUT --------- ####
        #pg.setConfigOption('background', 'w')
        #cwidget = EventWidget(self)
        cwidget = QtGui.QWidget()
        self.l0 = QtGui.QGridLayout()
        cwidget.setLayout(self.l0)
        self.setCentralWidget(cwidget)
        # ROI CHECKBOX
        self.checkBox = QtGui.QCheckBox('ROIs &On')
        self.checkBox.setStyleSheet("color: white;")
        self.checkBox.stateChanged.connect(self.ROIs_on)
        self.checkBox.toggle()
        self.l0.addWidget(self.checkBox,0,0,1,1)
        # number of ROIs in each image
        self.lcell0 = QtGui.QLabel('')
        self.lcell0.setStyleSheet("color: white;")
        self.lcell0.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.l0.addWidget(self.lcell0, 0,6,1,1)
        self.lcell1 = QtGui.QLabel('')
        self.lcell1.setStyleSheet("color: white;")
        self.l0.addWidget(self.lcell1, 0,10,1,1)
        # buttons to draw a square on view
        self.topbtns = QtGui.QButtonGroup()
        pos = [1,2,3,13,14,15]
        for b in range(6):
            btn  = gui.TopButton(b,self)
            self.topbtns.addButton(btn,b)
            self.l0.addWidget(btn, 0,pos[b],1,1)
            btn.setEnabled(False)
        self.topbtns.setExclusive(True)
        self.isROI=False
        self.ROIplot = 0
        # minimize view
        self.sizebtns = QtGui.QButtonGroup(self)
        b=0
        labels = ['cells','both','not cells']
        for l in labels:
            btn  = gui.SizeButton(b,l,self)
            self.sizebtns.addButton(btn,b)
            self.l0.addWidget(btn,0,7+b,1,1)
            btn.setEnabled(False)
            if b==1:
                btn.setEnabled(True)
            b+=1
        self.sizebtns.setExclusive(True)
        #### -------- MAIN PLOTTING AREA ---------- ####
        self.win = pg.GraphicsLayoutWidget()
        self.win.move(600,0)
        self.win.resize(1000,500)
        self.l0.addWidget(self.win,1,1,38,15)
        layout = self.win.ci.layout
        # --- cells image
        self.p1 = self.win.addViewBox(lockAspect=True,name='plot1',border=[100,100,100],
                                      row=0,col=0, invertY=True)
        self.img1 = pg.ImageItem(autoDownsample=True)
        self.p1.setMenuEnabled(False)
        data = np.zeros((700,512,3))
        self.img1.setImage(data)
        self.p1.addItem(self.img1)
        # --- noncells image
        self.p2 = self.win.addViewBox(lockAspect=True,name='plot2',border=[100,100,100],
                                      row=0,col=1, invertY=True)
        self.p2.setMenuEnabled(False)
        self.img2 = pg.ImageItem(autoDownsample=True)
        self.img2.setImage(data)
        self.p2.addItem(self.img2)
        self.p2.setXLink('plot1')
        self.p2.setYLink('plot1')
        # --- fluorescence trace plot
        self.p3 = self.win.addPlot(row=1,col=0,colspan=2)
        self.win.ci.layout.setRowStretchFactor(0,2)
        layout.setColumnMinimumWidth(0,1)
        layout.setColumnMinimumWidth(1,1)
        layout.setHorizontalSpacing(20)
        self.p3.setMouseEnabled(x=True,y=False)
        self.p3.enableAutoRange(x=True,y=True)
        self.win.scene().sigMouseClicked.connect(self.plot_clicked)
        #self.key_on(self.win.scene().keyPressEvent)
        self.show()
        self.win.show()
        #### --------- VIEW AND COLOR BUTTONS ---------- ####
        self.views = ['Q: ROIs', 'W: mean img (enhanced)', 'E: mean img', 'R: correlation map']
        self.colors = ['A: random', 'S: skew', 'D: compact','F: footprint','G: aspect_ratio','H: classifier','J: correlations']
        b = 0
        boldfont = QtGui.QFont("Arial", 10, QtGui.QFont.Bold)
        self.viewbtns = QtGui.QButtonGroup(self)
        vlabel = QtGui.QLabel(self)
        vlabel.setText("<font color='white'>Background</font>")
        vlabel.setFont(boldfont)
        vlabel.resize(vlabel.minimumSizeHint())
        self.l0.addWidget(vlabel,1,0,1,1)
        for names in self.views:
            btn  = gui.ViewButton(b,'&'+names,self)
            self.viewbtns.addButton(btn,b)
            self.l0.addWidget(btn,b+2,0,1,1)
            btn.setEnabled(False)
            b+=1
        self.viewbtns.setExclusive(True)
        # color buttons
        self.colorbtns = QtGui.QButtonGroup(self)
        clabel = QtGui.QLabel(self)
        clabel.setText("<font color='white'>Colors</font>")
        clabel.setFont(boldfont)
        self.l0.addWidget(clabel,b+3,0,1,1)
        nv = b+3
        b=0
        # colorbars for different statistics
        for names in self.colors:
            btn  = gui.ColorButton(b,'&'+names,self)
            self.colorbtns.addButton(btn,b)
            self.l0.addWidget(btn,nv+b+1,0,1,1)
            btn.setEnabled(False)
            self.colors[b] = self.colors[b][3:]
            b+=1
        self.bend = nv+b+4
        colorbarW = pg.GraphicsLayoutWidget()
        colorbarW.setMaximumHeight(60)
        colorbarW.setMaximumWidth(150)
        colorbarW.ci.layout.setRowStretchFactor(0,2)
        colorbarW.ci.layout.setContentsMargins(0,0,0,0)
        self.l0.addWidget(colorbarW, nv+b+2,0,1,1)
        self.colorbar = pg.ImageItem()
        cbar = colorbarW.addViewBox(row=0,col=0,colspan=3)
        cbar.setMenuEnabled(False)
        cbar.addItem(self.colorbar)
        self.clabel = [colorbarW.addLabel('0.0',color=[255,255,255],row=1,col=0),
                        colorbarW.addLabel('0.5',color=[255,255,255],row=1,col=1),
                        colorbarW.addLabel('1.0',color=[255,255,255],row=1,col=2)]
        self.probedit = QtGui.QDoubleSpinBox(self)
        self.probedit.setDecimals(3)
        self.probedit.setMaximum(1.0)
        self.probedit.setMinimum(0.0)
        self.probedit.setSingleStep(0.01)
        self.probedit.setValue(0.5)
        self.probedit.setFixedWidth(55)
        self.l0.addWidget(self.probedit,self.bend,0,1,1)
        plabel = QtGui.QLabel('\t    cell probability')
        plabel.setStyleSheet("color: white;")
        self.l0.addWidget(plabel,self.bend,0,1,1)
        self.applyclass = QtGui.QPushButton(' apply')
        self.applyclass.clicked.connect(lambda: classifier.apply(self))
        self.applyclass.setEnabled(False)
        self.applyclass.setStyleSheet(self.styleInactive)
        self.l0.addWidget(self.applyclass,self.bend+1,0,1,1)

        #### ----- CLASSIFIER BUTTONS ------- ####
        cllabel = QtGui.QLabel("")
        cllabel.setFont(boldfont)
        cllabel.setText("<font color='white'>Classifier</font>")
        self.classLabel = QtGui.QLabel("<font color='white'>load a classifier</font>")
        self.classLabel.setFont(QtGui.QFont('Arial',8))
        self.l0.addWidget(cllabel,self.bend+2,0,1,1)
        self.l0.addWidget(self.classLabel,self.bend+3,0,1,1)
        self.addtoclass = QtGui.QPushButton(' add current data to classifier')
        self.addtoclass.clicked.connect(lambda: classifier.add_to(self))
        self.addtoclass.setStyleSheet(self.styleInactive)
        self.l0.addWidget(self.addtoclass,self.bend+4,0,1,1)
        #### ------ CELL STATS -------- ####
        # which stats
        self.bend = self.bend+5
        self.stats_to_show = ['med','npix','skew','compact','footprint',
                              'aspect_ratio']
        lilfont = QtGui.QFont("Arial", 8)
        qlabel = QtGui.QLabel(self)
        qlabel.setFont(boldfont)
        qlabel.setText("<font color='white'>Selected ROI:</font>")
        self.l0.addWidget(qlabel,self.bend,0,1,1)
        self.ROIedit = QtGui.QLineEdit(self)
        self.ROIedit.setValidator(QtGui.QIntValidator(0,10000))
        self.ROIedit.setText('0')
        self.ROIedit.setFixedWidth(45)
        self.ROIedit.setAlignment(QtCore.Qt.AlignRight)
        self.ROIedit.returnPressed.connect(self.number_chosen)
        self.l0.addWidget(self.ROIedit, self.bend+1,0,1,1)
        self.ROIstats = []
        self.ROIstats.append(qlabel)
        for k in range(1,len(self.stats_to_show)+1):
            llabel = QtGui.QLabel(self.stats_to_show[k-1])
            self.ROIstats.append(llabel)
            self.ROIstats[k].setFont(lilfont)
            self.ROIstats[k].setStyleSheet("color: white;")
            self.ROIstats[k].resize(self.ROIstats[k].minimumSizeHint())
            self.l0.addWidget(self.ROIstats[k], self.bend+2+k,0,1,1)
        self.l0.addWidget(QtGui.QLabel(''), self.bend+3+k,0,1,1)
        self.l0.setRowStretch(self.bend+3+k, 1)
        # up/down arrows to resize view
        self.level = 1
        self.arrowButtons = [QtGui.QPushButton(u"  \u25b2"), QtGui.QPushButton(u"  \u25bc")]
        self.arrowButtons[0].clicked.connect(self.expand_trace)
        self.arrowButtons[1].clicked.connect(self.collapse_trace)
        b = 0
        for btn in self.arrowButtons:
            btn.setMaximumWidth(25)
            btn.setStyleSheet(self.styleUnpressed)
            self.l0.addWidget(btn, self.bend+4+k+b,0,1,1,QtCore.Qt.AlignRight)
            b+=1
        self.l0.addWidget(QtGui.QLabel(''), self.bend+6+k,0,1,1)
        # trace labels
        flabel = QtGui.QLabel(self)
        flabel.setText("<font color='blue'>Fluorescence</font>")
        nlabel = QtGui.QLabel(self)
        nlabel.setText("<font color='red'>Neuropil</font>")
        slabel = QtGui.QLabel(self)
        slabel.setText("<font color='gray'>Deconvolved</font>")
        flabel.setFont(boldfont)
        nlabel.setFont(boldfont)
        slabel.setFont(boldfont)
        flabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        nlabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        slabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
        self.l0.addWidget(flabel, self.bend+7+k,0,1,1)
        self.l0.addWidget(nlabel, self.bend+8+k,0,1,1)
        self.l0.addWidget(slabel, self.bend+9+k,0,1,1)
        # classifier file to load
        self.classfile = os.path.join(os.path.abspath(os.path.dirname(__file__)),
                         'classifiers/classifier_user.npy')
        self.classorig = os.path.join(os.path.abspath(os.path.dirname(__file__)),
                                          'classifiers/classifier.npy')
        model = np.load(self.classorig)
        model = model.item()
        self.default_keys = model['keys']
        #self.fname = '/media/carsen/DATA2/Github/TX4/stat.npy'
        #self.fname = 'C:/Users/carse/github/TX4/stat.npy'
        #self.load_proc()

    def keyPressEvent(self, event):
        if event.modifiers() !=  QtCore.Qt.ControlModifier:
            if event.key() == QtCore.Qt.Key_Return:
                merge=1
            elif event.key() == QtCore.Qt.Key_Escape:
                self.zoom_plot(1)
                self.show()
            elif event.key() == QtCore.Qt.Key_Delete:
                self.ROI_remove()
            elif event.key() == QtCore.Qt.Key_Shift:
                split=1
            elif event.key() == QtCore.Qt.Key_Q:
                self.viewbtns.button(0).setChecked(True)
                self.viewbtns.button(0).press(self, 0)
            elif event.key() == QtCore.Qt.Key_W:
                self.viewbtns.button(1).setChecked(True)
                self.viewbtns.button(1).press(self, 1)
            elif event.key() == QtCore.Qt.Key_E:
                self.viewbtns.button(2).setChecked(True)
                self.viewbtns.button(2).press(self, 2)
            elif event.key() == QtCore.Qt.Key_R:
                self.viewbtns.button(3).setChecked(True)
                self.viewbtns.button(3).press(self, 3)
            elif event.key() == QtCore.Qt.Key_O:
                self.checkBox.toggle()
            elif event.key() == QtCore.Qt.Key_A:
                self.colorbtns.button(0).setChecked(True)
                self.colorbtns.button(0).press(self, 0)
            elif event.key() == QtCore.Qt.Key_S:
                self.colorbtns.button(1).setChecked(True)
                self.colorbtns.button(1).press(self, 1)
            elif event.key() == QtCore.Qt.Key_D:
                self.colorbtns.button(2).setChecked(True)
                self.colorbtns.button(2).press(self, 2)
            elif event.key() == QtCore.Qt.Key_F:
                self.colorbtns.button(3).setChecked(True)
                self.colorbtns.button(3).press(self, 3)
            elif event.key() == QtCore.Qt.Key_G:
                self.colorbtns.button(4).setChecked(True)
                self.colorbtns.button(4).press(self, 4)
            elif event.key() == QtCore.Qt.Key_H:
                self.colorbtns.button(5).setChecked(True)
                self.colorbtns.button(5).press(self, 5)
            elif event.key() == QtCore.Qt.Key_J:
                self.colorbtns.button(6).setChecked(True)
                self.colorbtns.button(6).press(self, 6)
            #elif event.key() == QtCore.Qt.Key_K:
            #    self.colorbtns.button(7).press(self, 0)

    def expand_trace(self):
        self.level+=1
        self.level = np.minimum(5,self.level)
        self.win.ci.layout.setRowStretchFactor(1,self.level)

    def collapse_trace(self):
        self.level-=1
        self.level = np.maximum(1,self.level)
        self.win.ci.layout.setRowStretchFactor(1,self.level)

    def top_selection(self, bid):
        self.ROI_remove()
        ncells = len(self.stat)
        icells = np.minimum(ncells, 40)
        if bid==1:
            wplot=0
            top = True
        elif bid==2:
            wplot=0
            top = False
        elif bid==4:
            wplot=1
            top = True
        elif bid==5:
            wplot=1
            top=False
        if self.ops_plot[2] != 0:
            # correlation view
            if self.ops_plot[2]==self.ops_plot[3].shape[1]:
                istat = self.ops_plot[4]
            # statistics view
            else:
                istat = self.ops_plot[3][:,self.ops_plot[2]]
            if wplot==0:
                icell = np.array(self.iscell.nonzero()).flatten()
                istat = istat[self.iscell]
            else:
                icell = np.array((~self.iscell).nonzero()).flatten()
                istat = istat[~self.iscell]
            inds = istat.argsort()
            if top:
                inds = inds[:icells]
                self.ichosen = icell[inds[-1]]
            else:
                inds = inds[-icells:]
                self.ichosen = icell[inds[0]]
            self.imerge = []
            for n in inds:
                self.imerge.append(icell[n])
            # draw choices
            self.ichosen_stats()
            M = fig.draw_masks(self)
            fig.plot_masks(self,M)
            fig.plot_trace(self)
            self.show()

    def ROI_selection(self, wplot):
        if wplot==0:
            view = self.p1.viewRange()
        else:
            view = self.p2.viewRange()
        self.ROI_remove()
        self.ROIplot = int(np.floor(wplot/3))
        imx = (view[0][1] + view[0][0]) / 2
        imy = (view[1][1] + view[1][0]) / 2
        dx  = (view[0][1] - view[0][0]) / 4
        dy  = (view[1][1] - view[1][0]) / 4
        imx = imx - dx/2
        imy = imy - dy/2
        self.ROI = pg.RectROI([imx,imy],[dx,dy],pen='w',sideScalers=True)
        if wplot==0:
            self.p1.addItem(self.ROI)
        else:
            self.p2.addItem(self.ROI)
        self.ROI_position()
        self.ROI.sigRegionChangeFinished.connect(self.ROI_position)
        self.isROI = True

    def ROI_remove(self):
        if self.isROI:
            if self.ROIplot==0:
                self.p1.removeItem(self.ROI)
            else:
                self.p2.removeItem(self.ROI)
            self.isROI=False
            self.topbtns.button(0).setStyleSheet(self.styleUnpressed)
            self.topbtns.button(3).setStyleSheet(self.styleUnpressed)

    def ROI_position(self):
        pos0 = self.ROI.getSceneHandlePositions()
        if self.ROIplot==0:
            pos = self.p1.mapSceneToView(pos0[0][1])
        else:
            pos = self.p2.mapSceneToView(pos0[0][1])
        posy = pos.y()
        posx = pos.x()
        sizex,sizey = self.ROI.size()
        xrange = (np.arange(-1*int(sizex),1) + int(posx)).astype(np.int32)
        yrange = (np.arange(-1*int(sizey),1) + int(posy)).astype(np.int32)
        xrange = xrange[xrange>=0]
        xrange = xrange[xrange<self.ops['Lx']]
        yrange = yrange[yrange>=0]
        yrange = yrange[yrange<self.ops['Ly']]
        ypix,xpix = np.meshgrid(yrange,xrange)
        self.select_cells(ypix,xpix)

    def number_chosen(self):
        if self.loaded:
            self.ichosen = int(self.ROIedit.text())
            if self.ichosen >= len(self.stat):
                self.ichosen = len(self.stat) - 1
            self.imerge = [self.ichosen]
            if self.ops_plot[2]==self.ops_plot[3].shape[1]:
                fig.corr_masks(self)
                fig.plot_colorbar(self, self.ops_plot[2])
            self.ichosen_stats()
            M = fig.draw_masks(self)
            fig.plot_masks(self,M)
            fig.plot_trace(self)
            self.show()

    def select_cells(self,ypix,xpix):
        i = self.ROIplot
        iROI0 = self.iROI[i,0,ypix,xpix]
        icells = np.unique(iROI0[iROI0>=0])
        self.imerge = []
        for n in icells:
            if (self.iROI[i,:,ypix,xpix]==n).sum()>0.6*self.stat[n]['npix']:
                self.imerge.append(n)
        if len(self.imerge)>0:
            self.ichosen = self.imerge[0]
            if self.ops_plot[2]==self.ops_plot[3].shape[1]:
                fig.corr_masks(self)
                fig.plot_colorbar(self, self.ops_plot[2])
            self.ichosen_stats()
            M = fig.draw_masks(self)
            fig.plot_masks(self,M)
            fig.plot_trace(self)
            self.show()

    def make_masks_and_buttons(self):
        self.ROI_remove()
        self.isROI=False
        self.ops_plot[1] = 0
        self.ops_plot[2] = 0
        self.ops_plot[3] = []
        self.ops_plot[4] = []
        self.setWindowTitle(self.fname)
        # add boundaries to stat for ROI overlays
        ncells = len(self.stat)
        for n in range(0,ncells):
            ypix = self.stat[n]['ypix'].flatten()
            xpix = self.stat[n]['xpix'].flatten()
            iext = fig.boundary(ypix,xpix)
            self.stat[n]['yext'] = ypix[iext]
            self.stat[n]['xext'] = xpix[iext]
            ycirc,xcirc = fig.circle(self.stat[n]['med'],self.stat[n]['radius'])
            goodi = (ycirc>=0) & (xcirc>=0) & (ycirc<self.ops['Ly']) & (xcirc<self.ops['Lx'])
            self.stat[n]['ycirc'] = ycirc[goodi]
            self.stat[n]['xcirc'] = xcirc[goodi]
        # enable buttons
        self.enable_views_and_classifier()
        # make color arrays for various views
        fig.make_colors(self)
        self.ichosen = int(0)
        self.imerge = [int(0)]
        self.iflip = int(0)
        self.ichosen_stats()
        # colorbar
        self.colormat = fig.make_colorbar()
        fig.plot_colorbar(self, self.ops_plot[2])
        tic=time.time()
        fig.init_masks(self)
        print(time.time()-tic)
        fig.corr_masks(self)
        M = fig.draw_masks(self)
        fig.plot_masks(self,M)
        self.lcell1.setText('%d'%(ncells-self.iscell.sum()))
        self.lcell0.setText('%d'%(self.iscell.sum()))
        fig.init_range(self)
        fig.plot_trace(self)
        if (type(self.ops['diameter']) is not int) and (len(self.ops['diameter'])>1):
            self.xyrat = self.ops['diameter'][0] / self.ops['diameter'][1]
        else:
            self.xyrat = 1.0
        self.p1.setAspectLocked(lock=True, ratio=self.xyrat)
        self.p2.setAspectLocked(lock=True, ratio=self.xyrat)
        self.show()
        # no classifier loaded
        classifier.activate(self, False)

    def enable_views_and_classifier(self):
        for b in range(len(self.views)):
            self.viewbtns.button(b).setEnabled(True)
            self.viewbtns.button(b).setStyleSheet(self.styleUnpressed)
            #self.viewbtns.button(b).setShortcut(QtGui.QKeySequence('R'))
            if b==0:
                self.viewbtns.button(b).setChecked(True)
                self.viewbtns.button(b).setStyleSheet(self.stylePressed)
        for b in range(len(self.colors)):
            self.colorbtns.button(b).setEnabled(True)
            self.colorbtns.button(b).setStyleSheet(self.styleUnpressed)
            if b==0:
                self.colorbtns.button(b).setChecked(True)
                self.colorbtns.button(b).setStyleSheet(self.stylePressed)
        for b in [0,3]:
            self.topbtns.button(b).setStyleSheet(self.styleUnpressed)
            self.topbtns.button(b).setEnabled(True)
        self.applyclass.setStyleSheet(self.styleUnpressed)
        self.applyclass.setEnabled(True)
        b = 0
        for btn in self.sizebtns.buttons():
            btn.setStyleSheet(self.styleUnpressed)
            btn.setEnabled(True)
            if b==1:
                btn.setChecked(True)
                btn.setStyleSheet(self.stylePressed)
            b+=1
        # enable classifier menu
        self.loadClass.setEnabled(True)
        self.loadTrain.setEnabled(True)
        self.loadUClass.setEnabled(True)
        self.loadSClass.setEnabled(True)
        self.resetDefault.setEnabled(True)
        self.visualizations.setEnabled(True)

    def ROIs_on(self,state):
        if state == QtCore.Qt.Checked:
            self.ops_plot[0] = True
        else:
            self.ops_plot[0] = False
        if self.loaded:
            M = fig.draw_masks(self)
            fig.plot_masks(self,M)

    def plot_clicked(self,event):
        '''left-click chooses a cell, right-click flips cell to other view'''
        flip = False
        choose = False
        zoom = False
        replot = False
        items = self.win.scene().items(event.scenePos())
        posx  = 0
        posy  = 0
        iplot = 0
        if self.loaded:
            #print(event.modifiers() == QtCore.Qt.ControlModifier)
            for x in items:
                if x==self.img1:
                    pos = self.p1.mapSceneToView(event.scenePos())
                    posy = pos.x()
                    posx = pos.y()
                    iplot = 1
                elif x==self.img2:
                    pos = self.p2.mapSceneToView(event.scenePos())
                    posy = pos.x()
                    posx = pos.y()
                    iplot = 2
                elif x==self.p3:
                    iplot = 3
                elif (x==self.p1 or x==self.p2) and x!=self.img1 and x!=self.img2:
                    iplot = 4
                    if event.double():
                        zoom = True
                if iplot==1 or iplot==2:
                    if event.button()==2:
                        flip = True
                    elif event.button()==1:
                        if event.double():
                            zoom = True
                        else:
                            choose = True
                if iplot==3 and event.double():
                    zoom = True
                posy = int(posy)
                posx = int(posx)
                if zoom:
                    self.zoom_plot(iplot)
                if (choose or flip) and (iplot==1 or iplot==2):
                    ichosen = int(self.iROI[iplot-1,0,posx,posy])
                    if ichosen<0:
                        choose = False
                        flip   = False
                if choose:
                    merged = False
                    if event.modifiers() == QtCore.Qt.ControlModifier:
                        if self.iscell[self.imerge[0]] == self.iscell[ichosen]:
                            if ichosen not in self.imerge:
                                self.imerge.append(ichosen)
                                self.ichosen = ichosen
                                merged = True
                            elif ichosen in self.imerge and len(self.imerge)>1:
                                self.imerge.remove(ichosen)
                                self.ichosen = self.imerge[0]
                                merged = True
                    if not merged:
                        self.imerge = [ichosen]
                        self.ichosen = ichosen
                if flip:
                    if ichosen not in self.imerge:
                        self.imerge = [ichosen]
                        self.ichosen = ichosen
                    self.flip_plot(iplot)
                if choose or flip or replot:
                    if self.isROI:
                        self.ROI_remove()
                    for btn in self.topbtns.buttons():
                        btn.setStyleSheet(self.styleUnpressed)
                    if self.ops_plot[2]==self.ops_plot[3].shape[1]:
                        fig.corr_masks(self)
                        fig.plot_colorbar(self, self.ops_plot[2])
                    self.ichosen_stats()
                    M = fig.draw_masks(self)
                    fig.plot_masks(self,M)
                    fig.plot_trace(self)
                    self.show()
                    #print(time.time()-tic)

    def ichosen_stats(self):
        n = self.ichosen
        self.ROIedit.setText(str(self.ichosen))
        for k in range(1,len(self.stats_to_show)+1):
            key = self.stats_to_show[k-1]
            ival = self.stat[n][key]
            if k==1:
                self.ROIstats[k].setText(key+': [%d, %d]'%(ival[0],ival[1]))
            elif k==2:
                self.ROIstats[k].setText(key+': %d'%(ival))
            else:
                self.ROIstats[k].setText(key+': %2.2f'%(ival))

    def flip_plot(self,iplot):
        self.iflip = self.ichosen
        for n in self.imerge:
            iscell = int(self.iscell[n])
            self.iscell[n] = ~self.iscell[n]
            self.ichosen = n
            fig.flip_cell(self)
        np.save(self.basename+'/iscell.npy',
                np.concatenate((np.expand_dims(self.iscell,axis=1),
                np.expand_dims(self.probcell,axis=1)), axis=1))
        self.lcell0.setText('%d'%(self.iscell.sum()))
        self.lcell1.setText('%d'%(self.iscell.size-self.iscell.sum()))

    def zoom_plot(self,iplot):
        if iplot==1 or iplot==2 or iplot==4:
            self.p1.setXRange(0,self.ops['Lx'])
            self.p1.setYRange(0,self.ops['Ly'])
            self.p2.setXRange(0,self.ops['Lx'])
            self.p2.setYRange(0,self.ops['Ly'])
        else:
            self.p3.setXRange(0,self.Fcell.shape[1])
            self.p3.setYRange(self.fmin,self.fmax)
        self.show()

    def run_suite2p(self):
        RW = gui.RunWindow(self)
        RW.show()

    def vis_window(self):
        VW = visualize.VisWindow(self)
        VW.show()

    def load_dialog(self):
        name = QtGui.QFileDialog.getOpenFileName(self, 'Open stat.npy', filter='stat.npy')
        self.fname = name[0]
        self.load_proc()

    def load_proc(self):
        name = self.fname
        print(name)
        try:
            stat = np.load(name)
            ypix = stat[0]['ypix']
        except (ValueError, KeyError, OSError, RuntimeError, TypeError, NameError):
            print('ERROR: this is not a stat.npy file :( (needs stat[n]["ypix"]!)')
            stat = None
        if stat is not None:
            basename, fname = os.path.split(name)
            goodfolder = True
            try:
                Fcell = np.load(basename + '/F.npy')
                Fneu = np.load(basename + '/Fneu.npy')
            except (ValueError, OSError, RuntimeError, TypeError, NameError):
                print('ERROR: there are no fluorescence traces in this folder (F.npy/Fneu.npy)')
                goodfolder = False
            try:
                Spks = np.load(basename + '/spks.npy')
            except (ValueError, OSError, RuntimeError, TypeError, NameError):
                print('there are no spike deconvolved traces in this folder (spks.npy)')
            try:
                iscell = np.load(basename + '/iscell.npy')
                probcell = iscell[:,1]
                iscell = iscell[:,0].astype(np.bool)
            except (ValueError, OSError, RuntimeError, TypeError, NameError):
                print('no manual labels found (iscell.npy)')
            try:
                ops = np.load(basename + '/ops.npy')
                ops = ops.item()
            except (ValueError, OSError, RuntimeError, TypeError, NameError):
                print('ERROR: there is no ops file in this folder (ops.npy)')
                goodfolder = False
            if goodfolder:
                self.basename = basename
                self.stat = stat
                self.ops  = ops
                self.Fcell = Fcell
                self.Fneu = Fneu
                self.Spks = Spks
                self.iscell = iscell
                self.probcell = probcell
                self.make_masks_and_buttons()
                self.loaded = True
            else:
                print('stat.npy found, but other files not in folder')
                Text = 'stat.npy found, but other files missing, choose another?'
                self.load_again(Text)
        else:
            Text = 'Incorrect file, not a stat.npy, choose another?'
            self.load_again(Text)

    def load_again(self,Text):
        tryagain = QtGui.QMessageBox.question(self, 'ERROR',
                                        Text,
                                        QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        if tryagain == QtGui.QMessageBox.Yes:
            self.load_dialog()

    def load_classifier(self):
        name = QtGui.QFileDialog.getOpenFileName(self, 'Open File')
        if name:
            classifier.load(self, name[0])
            self.class_activated()
        else:
            print('no classifier')

    def load_s2p_classifier(self):
        classifier.load(self, self.classorig)
        self.class_file()
        self.saveDefault.setEnabled(True)

    def load_default_classifier(self):
        classifier.load(self, os.path.join(os.path.abspath(os.path.dirname(__file__)),
                         'classifiers/classifier_user.npy'))
        self.class_activated()

    def class_file(self):
        if self.classfile == os.path.join(os.path.abspath(os.path.dirname(__file__)),
                         'classifiers/classifier_user.npy'):
            cfile = 'default classifier'
        elif self.classfile == os.path.join(os.path.abspath(os.path.dirname(__file__)),
                             'classifiers/classifier.npy'):
            cfile = 'suite2p classifier'
        else:
            cfile = self.classfile
        cstr = "<font color='white'>" + cfile + "</font>"
        self.classLabel.setText(cstr)

    def class_activated(self):
        self.class_file()
        self.saveDefault.setEnabled(True)
        self.addtoclass.setStyleSheet(self.styleUnpressed)
        self.addtoclass.setEnabled(True)

    def class_default(self):
        dm = QtGui.QMessageBox.question(self,'Default classifier',
                                        'Are you sure you want to overwrite your default classifier?',
                                        QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        if dm == QtGui.QMessageBox.Yes:
            classfile = os.path.join(os.path.abspath(os.path.dirname(__file__)),
                             'classifiers/classifier_user.npy')
            np.save(classfile, self.model)

    def reset_default(self):
        dm = QtGui.QMessageBox.question(self,'Default classifier',
                                        'Are you sure you want to reset the default classifier to the built-in suite2p classifier?',
                                        QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        if dm == QtGui.QMessageBox.Yes:
            classfile = os.path.join(os.path.abspath(os.path.dirname(__file__)),
                             'classifiers/classifier_user.npy')
            shutil.copy(self.classorig, classfile)
    #def save_gui_data(self):
    #    gui_data = {
    #                'RGBall': self.RGBall,
    #                'RGBback': self.RGBback,
    #                'Vback': self.Vback,
    #                'iROI': self.iROI,
    #                'iExt': self.iExt,
    #                'Sroi': self.Sroi,
    #                'Sext': self.Sext,
    #                'Lam': self.Lam,
    #                'LamMean': self.LamMean,
    #                'wasloaded': True
    #               }
    #    np.save(self.basename+'/gui_data.npy', gui_data)

def run():
    ## Always start by initializing Qt (only once per application)
    app = QtGui.QApplication(sys.argv)
    icon_path = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                    'logo/logo.png')
    app_icon = QtGui.QIcon()
    app_icon.addFile(icon_path, QtCore.QSize(16,16))
    app_icon.addFile(icon_path, QtCore.QSize(24,24))
    app_icon.addFile(icon_path, QtCore.QSize(32,32))
    app_icon.addFile(icon_path, QtCore.QSize(48,48))
    app_icon.addFile(icon_path, QtCore.QSize(96,96))
    app_icon.addFile(icon_path, QtCore.QSize(256,256))
    app.setWindowIcon(app_icon)
    GUI = MainW()
    ret = app.exec_()
    #GUI.save_gui_data()
    sys.exit(ret)

#run()



from PyQt5 import QtGui, QtCore
from PyQt5.QtWidgets import QApplication, QMainWindow, QTableWidget

import matplotlib
matplotlib.use("Qt5Agg")
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5 import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
import matplotlib.pyplot as plt

import pyqtgraph as pg
import sys
import numpy as np

def newwindow():
    LoadW = QtGui.QWindow()
    LoadW.show()


    
    
class MainW(QtGui.QMainWindow):
    resized = QtCore.pyqtSignal()
    def __init__(self):
        super(MainW, self).__init__()
        self.setGeometry(50,50,1600,1000)
        self.setWindowTitle('suite2p')
        #self.setStyleSheet("QMainWindow {background: 'black';}")
        self.selectionMode = False
        self.masks = np.random.random((512,512,3))
        self.resized.connect(self.windowsize)
        ### menu bar options
        # load processed data
        loadProc = QtGui.QAction('&Load processed data', self)
        loadProc.setShortcut('Ctrl+L')
        loadProc.setStatusTip('load processed data in suite2p format')
        loadProc.triggered.connect(self.load_proc)
        # load masks
        loadMask = QtGui.QAction('&Load masks and extract traces', self)
        loadMask.setShortcut('Ctrl+L')
        loadMask.setStatusTip('load mask pixels in suite2p format')
        # save file
        saveFile = QtGui.QAction('&Save', self)
        saveFile.setShortcut('Ctrl+S')
        saveFile.triggered.connect(self.file_save)
        # make menuBar!
        main_menu = self.menuBar()
        file_menu = main_menu.addMenu('&File')
        file_menu.addAction(loadProc)
        file_menu.addAction(loadMask)
        file_menu.addAction(saveFile)

        ### main widget with plots
        #self.main_widget = QtGui.QWidget(self)
        #self.main_widget.resize(500,500)
        #self.main_widget.move(300,300)
        wid = QtGui.QWidget(self)
        self.setCentralWidget(wid)
        l = QtGui.QVBoxLayout()
        wid.setLayout(l)
        #self.plot = pg.GraphicsView()
        self.plot = PlotCells(self, dpi=100)
        self.toolbar = NavigationToolbar(self.plot,self)
        #self.toolbar.Realize()
        #tw,th=self.toolbar.GetSizeTuple()
        #fw,fh=self.plot.GetSizeTuple()
        self.toolbar.resize(500,30)
        self.toolbar.move(300,0)
        
        cid = self.plot.mpl_connect('button_press_event', self.plot.onclick)
        #self.toolbar = NavigationToolbar(self.sc, self.main_widget)
        #dc = MyDynamicMplCanvas(self.main_widget, width=5, height=4, dpi=100)
        l.addWidget(self.plot)
        #self.setLayout(self.l)
        #self.setCentralWidget(l)
        #l.addWidget(dc)
        #self.main_widget.setFocus()
        #self.setCentralWidget(self.main_widget)

        
        ### checkboxes
        checkBox = QtGui.QCheckBox('plot neuropil',self)
        checkBox.move(100,100)
        checkBox.stateChanged.connect(self.plot_neuropil)
        checkBox.toggle()

        #self.plots = QtGui.QWidget(self,width=1000,height=300)
        #layout = QtGui.QGridLayout()
        #self.plots.setLayout(layout)
        #plot1 = pg.PlotWidget
        #layout.addWidget(plot1,0,0,1,1)
        #elf.plots =  QtGui.QWidget(self,width=1000,height=300)
        #elf.plots.move(300,100)
        #self.plots        = PlotCells(self,width=10,height=4)


        #layout = QtGui.QVBoxLayout()
        #layout.addWidget(self.mpl_toolbar)
        #layout.addWidget(self.plots)
        #self.setLayout(layout)
        
        #elf.plots.addWidget(self.navi_toolbar)


        ### buttons for different views
        self.mask_view()

    def windowsize(self):
        print(10)
    #def resizeEvent(self,event):
    #    self.resized.emit()
    #    return super(MainW,self).resizeEvent(event)

    def plot_neuropil(self,state):
        if state == QtCore.Qt.Checked:
            print('yay')
        else:
            print('boo')

    def load_proc(self):
        name = QtGui.QFileDialog.getOpenFileName(self, 'Open File')
        #print(name)
        masks = np.load(name[0])

        if masks.ndim == 2 | (masks.ndim==3 & masks.shape[2]==3):
            self.masks = masks
            self.plot.plot(masks,masks)
            #pg.image(masks)
            self.selectionMode = True
        else:        
            tryagain = QtGui.QMessageBox.question(self, 'error',
                                                  'Incorrect file, choose another?',
                                                  QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
            if tryagain == QtGui.QMessageBox.Yes:
                self.load_proc(self)
            else:
                pass

    def file_save(self):
        name = QtGui.QFileDialog.getSaveFileName(self,'Save File')
        file = open(name,'w')
        file.write('boop')
        file.close()
        
    # different mask views
    def mask_view(self):
        btn = QtGui.QPushButton('mean image (M)', self)
        btn.setShortcut('M')
        btn.clicked.connect(newwindow)
        btn.resize(btn.minimumSizeHint())
        btn.move(10,30)
        self.show()

class PlotCells(FigureCanvas):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        #self.axes = fig.add_subplot(111)
        
        FigureCanvas.__init__(self, fig)
        self.setParent(parent)
        self.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.setFocus()
        #self.mpl_toolbar = NavigationToolbar(self, parent)
        FigureCanvas.setSizePolicy(self,
                                   QtGui.QSizePolicy.Expanding,
                                   QtGui.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

        # add axes
        self.figure.patch.set_facecolor('black')
        self.ax1 = self.figure.add_axes([0.01,0.2,0.47,0.73])
        self.ax1.axis('off')
        self.ax2 = self.figure.add_axes([0.52,0.2,0.47,0.73], sharex=self.ax1, sharey=self.ax1)
        self.ax2.axis('off')
        self.ax3 = self.figure.add_axes([0.01,0.02,0.98,0.2])
        self.ax3.set_facecolor('black')
        
    def plot(self, masks1, masks2):
        self.ax1.imshow(masks1)
        self.ax2.imshow(masks2)
        self.ax3.plot(np.random.random((1000,)))

        #ax.set_title('PyQt Matplotlib Example')
        #self.figure.tight_layout()
                
        self.draw()
        self.show()

    def onclick(self,event):
        if event.inaxes:
            print('%s click: button=%d, x=%d, y=%d, xdata=%f, ydata=%f' %
                  ('double' if event.dblclick else 'single', event.button,
                   event.x, event.y, event.xdata, event.ydata))
            if self.ax1 is event.inaxes:
                print('plot1')
    
            
def run():
    ## Always start by initializing Qt (only once per application)
    app = QtGui.QApplication(sys.argv)
    GUI = MainW()
    #plot = pg.PlotWidget()
    #GUI.setCentralWidget(plot)
    #plot.resize(400,400)
    #plot.sigPointsClicked.connect(plot,meclick)
    sys.exit(app.exec_())
    
run()

## Define a top-level widget to hold everything
#w = QtGui.QWidget()
#w.setGeometry(0,0,900,500)
#w.setWindowTitle('suite2p')

## Create some widgets to be placed inside
#btn = QtGui.QPushButton('press me')
#text = QtGui.QLineEdit('enter text')
#listw = QtGui.QListWidget()
#plot = pg.PlotWidget()

## Create a grid layout to manage the widgets size and position
#layout = QtGui.QGridLayout()
#w.setLayout(layout)

## Add widgets to the layout in their proper positions
#layout.addWidget(btn, 0, 0)   # button goes in upper-left
#layout.addWidget(text, 1, 0)   # text edit goes in middle-left
#layout.addWidget(listw, 2, 0)  # list widget goes in bottom-left
#layout.addWidget(plot, 0, 1, 3, 1)  # plot goes on right side, spanning 3 rows

## Display the widget as a new window
#w.show()

## Start the Qt event loop
#app.exec_()



from PyQt5 import QtGui, QtCore
import pyqtgraph as pg
import sys
import numpy as np
import os
import pickle
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
import matplotlib.pyplot as plt
import fig
import gui
import classifier
import time
class MainW(QtGui.QMainWindow):
    def __init__(self):
        super(MainW, self).__init__()
        self.setGeometry(25,25,1600,1000)
        self.setWindowTitle('suite2p')
        #self.setStyleSheet("QMainWindow {background: 'black';}")
        self.loaded = False
        self.ops_plot = []
        # default plot options
        self.ops_plot.append(True)
        self.ops_plot.append(0)
        self.ops_plot.append(0)
        ### menu bar options
        # run suite2p from scratch
        runS2P =  QtGui.QAction('&Run suite2p ', self)
        runS2P.setShortcut('Ctrl+R')
        runS2P.triggered.connect(self.run_suite2p)
        self.addAction(runS2P)
        # load processed data
        loadProc = QtGui.QAction('&Load processed data (choose stat.npy file)', self)
        loadProc.setShortcut('Ctrl+L')
        loadProc.triggered.connect(self.load_dialog)
        self.addAction(loadProc)
        # load masks
        #loadMask = QtGui.QAction('&Load masks (stat.pkl) and extract traces', self)
        #loadMask.setShortcut('Ctrl+M')
        #self.addAction(loadMask)
        # make mainmenu!
        main_menu = self.menuBar()
        file_menu = main_menu.addMenu('&File')
        file_menu.addAction(runS2P)
        file_menu.addAction(loadProc)
        #file_menu.addAction(loadMask)
        # classifier menu
        self.trainfiles = []
        self.statlabels = None
        self.statclass = ['skew', 'compact', 'footprint']
        self.loadClass = QtGui.QAction('&Load classifier', self)
        self.loadClass.setShortcut('Ctrl+K')
        self.loadClass.triggered.connect(self.load_classifier)
        self.loadClass.setEnabled(False)
        self.loadTrain = QtGui.QAction('&Train classifier (choose iscell.npy files)', self)
        self.loadTrain.setShortcut('Ctrl+T')
        self.loadTrain.triggered.connect(self.load_traindata)
        self.loadTrain.setEnabled(False)
        self.saveClass = QtGui.QAction('&Save classifier', self)
        self.saveClass.setShortcut('Ctrl+S')
        self.saveClass.triggered.connect(self.save_classifier)
        self.saveClass.setEnabled(False)
        self.saveTrain = QtGui.QAction('&Save training list', self)
        #self.saveTrain.setShortcut('Ctrl+S')
        self.saveTrain.triggered.connect(self.save_trainlist)
        self.saveTrain.setEnabled(False)
        class_menu = main_menu.addMenu('&Classifier')
        class_menu.addAction(self.loadClass)
        class_menu.addAction(self.loadTrain)
        class_menu.addAction(self.saveClass)
        class_menu.addAction(self.saveTrain)

        # main widget
        cwidget = QtGui.QWidget(self)
        self.l0 = QtGui.QGridLayout()
        cwidget.setLayout(self.l0)
        self.setCentralWidget(cwidget)
        # ROI CHECKBOX
        checkBox = QtGui.QCheckBox('ROIs on')
        checkBox.move(30,100)
        checkBox.stateChanged.connect(self.ROIs_on)
        checkBox.toggle()
        self.l0.addWidget(checkBox,0,0,1,1)
        # MAIN PLOTTING AREA
        self.win = pg.GraphicsView()
        self.win.move(600,0)
        self.win.resize(1000,500)
        self.l0.addWidget(self.win,0,1,18,12)
        l = pg.GraphicsLayout(border=(100,100,100))
        self.win.setCentralItem(l)
        self.p0 = l.addLabel('load a stat.pkl file',row=0,col=0,colspan=2)
        # cells image
        self.p1 = l.addViewBox(lockAspect=True,name='plot1',row=1,col=0)
        self.img1 = pg.ImageItem()
        self.p1.setMenuEnabled(False)
        data = np.zeros((700,512,3))
        self.img1.setImage(data)
        self.p1.addItem(self.img1)
        #self.p1.setXRange(0,512,padding=0.25)
        #self.p1.setYRange(0,512,padding=0.25)
        # noncells image
        self.p2 = l.addViewBox(lockAspect=True,name='plot2',row=1,col=1)
        self.p2.setMenuEnabled(False)
        self.img2 = pg.ImageItem()
        self.img2.setImage(data)
        self.p2.addItem(self.img2)
        #self.p2.autoRange()
        self.p2.setXLink('plot1')
        self.p2.setYLink('plot1')
        # fluorescence trace plot
        self.p3 = l.addPlot(row=2,col=0,colspan=2)
        #x = np.arange(0,20000)
        #y = np.zeros((20000,))
        #self.p3.clear()
        #self.p3.plot(x,y,pen='b')
        self.p3.setMouseEnabled(x=True,y=False)
        self.p3.enableAutoRange(x=True,y=True)
        self.win.scene().sigMouseClicked.connect(self.plot_clicked)

        self.show()
        self.win.show()
        #
        #self.load_proc(['/media/carsen/DATA2/Github/data/stat.pkl','*'])
        self.fname = 'C:/Users/carse/github/data/stat.npy'
        self.load_proc()

    def make_masks_and_buttons(self, name):
        self.p0.setText(name)
        views = ['mean img', 'correlation map']
        # add boundaries to stat for ROI overlays
        ncells = self.Fcell.shape[0]
        for n in range(0,ncells):
            ypix = self.stat[n]['ypix']
            xpix = self.stat[n]['xpix']
            iext = np.expand_dims(fig.boundary(ypix,xpix),axis=0)
            self.stat[n]['yext'] = ypix[iext]
            self.stat[n]['xext'] = xpix[iext]

        if 'mean_image_red' in self.ops:
            views.append('red channel mean')
        colors = ['random', 'skew', 'compact','footprint',
                    'aspect_ratio']
        b = 0
        self.viewbtns = QtGui.QButtonGroup(self)
        vlabel = QtGui.QLabel(self)
        vlabel.setText('Background')
        vlabel.resize(vlabel.minimumSizeHint())
        self.l0.addWidget(vlabel,1,0,1,1)
        self.btnstate = []
        for names in views:
            btn  = gui.ViewButton(b,names,self)
            self.viewbtns.addButton(btn,b)
            self.l0.addWidget(btn,b+2,0,1,1)
            self.btnstate.append(False)
            b+=1
        self.colorbtns = QtGui.QButtonGroup(self)
        clabel = QtGui.QLabel(self)
        clabel.setText('Colors')
        clabel.resize(clabel.minimumSizeHint())
        self.l0.addWidget(clabel,b+2,0,1,1)
        nv = b+2
        b=0
        allcols = np.random.random((ncells,1))
        self.clabels = []
        # colorbars for different statistics
        self.colorfig = plt.figure(figsize=(1,0.05))
        self.canvas = FigureCanvas(self.colorfig)
        self.colorbar = self.colorfig.add_subplot(211)
        for names in colors:
            if names in self.stat[0] or b==0:
                if b > 0:
                    istat = np.zeros((ncells,1))
                    for n in range(0,ncells):
                        istat[n] = self.stat[n][names]
                    self.clabels.append([istat.min(), (istat.max()-istat.min())/2, istat.max()])
                    istat = istat - istat.min()
                    istat = istat / istat.max()
                    istat = istat / 1.3
                    istat = istat + 0.1
                    icols = 1 - istat
                    allcols = np.concatenate((allcols, icols), axis=1)
                else:
                    self.clabels.append([0,0.5,1])
                btn  = gui.ColorButton(b,names,self)
                self.colorbtns.addButton(btn,b)
                self.l0.addWidget(btn,nv+b+1,0,1,1)
                self.btnstate.append(False)
                if b==0:
                    btn.setChecked(True)
                b+=1
        self.classbtn  = gui.ColorButton(b,'classifier',self)
        self.colorbtns.addButton(self.classbtn,b)
        self.ncolors = b+1
        self.classbtn.setEnabled(False)
        self.l0.addWidget(self.classbtn,nv+b+1,0,1,1)
        self.btnstate.append(False)
        self.ops_plot.append(allcols)
        self.iROI = fig.ROI_index(self.ops, self.stat)
        self.ichosen = int(0)
        self.iflip = int(0)
        if not hasattr(self, 'iscell'):
            self.iscell = np.ones((ncells,), dtype=bool)
        fig.init_masks(self)
        M = fig.draw_masks(self)
        self.plot_masks(M)
        self.l0.addWidget(self.canvas,nv+b+2,0,1,1)
        self.colormat = fig.make_colorbar()
        self.plot_colorbar(0)
        #gl = pg.GradientLegend((10,300),(10,30))
        #gl.setParentItem(self.p1)
        self.p1.setXRange(0,self.ops['Ly'])
        self.p1.setYRange(0,self.ops['Lx'])
        self.p2.setXRange(0,self.ops['Ly'])
        self.p2.setYRange(0,self.ops['Lx'])
        self.p3.setLimits(xMin=0,xMax=self.Fcell.shape[1])
        self.trange = np.arange(0, self.Fcell.shape[1])
        self.plot_trace()
        self.loadClass.setEnabled(True)
        self.loadTrain.setEnabled(True)
        self.show()

    def plot_colorbar(self, bid):
        self.colorbar.clear()
        if bid==0:
            self.colorbar.imshow(np.zeros((20,100,3)))
        else:
            self.colorbar.imshow(self.colormat)
        self.colorbar.tick_params(axis='y',which='both',left=False,right=False,
                                labelleft=False,labelright=False)
        self.colorbar.set_xticks([0,50,100])
        self.colorbar.set_xticklabels(['%1.2f'%self.clabels[bid][0],
                                        '%1.2f'%self.clabels[bid][1],
                                        '%1.2f'%self.clabels[bid][2]],
                                        fontsize=6)
        self.canvas.draw()

    def plot_trace(self):
        self.p3.clear()
        self.p3.plot(self.trange,self.Fcell[self.ichosen,:],pen='b')
        self.p3.plot(self.trange,self.Fneu[self.ichosen,:],pen='r')
        self.fmax = np.maximum(self.Fcell[self.ichosen,:].max(), self.Fneu[self.ichosen,:].max())
        self.fmin = np.minimum(self.Fcell[self.ichosen,:].min(), self.Fneu[self.ichosen,:].min())
        self.p3.setXRange(0,self.Fcell.shape[1])
        self.p3.setYRange(self.fmin,self.fmax)

    def ROIs_on(self,state):
        if state == QtCore.Qt.Checked:
            self.ops_plot[0] = True
        else:
            self.ops_plot[0] = False
        if self.loaded:
            M = fig.draw_masks(self)
            self.plot_masks(M)

    def plot_masks(self,M):
        self.img1.setImage(M[0])
        self.img2.setImage(M[1])
        self.img1.show()
        self.img2.show()

    def plot_clicked(self,event):
        '''left-click chooses a cell, right-click flips cell to other view'''
        flip = False
        choose = False
        zoom = False
        items = self.win.scene().items(event.scenePos())
        posx  = 0
        posy  = 0
        iplot = 0
        if self.loaded:
            for x in items:
                if x==self.img1:
                    pos = self.p1.mapSceneToView(event.scenePos())
                    posx = pos.x()
                    posy = pos.y()
                    iplot = 1
                elif x==self.img2:
                    pos = self.p2.mapSceneToView(event.scenePos())
                    posx = pos.x()
                    posy = pos.y()
                    iplot = 2
                elif x==self.p3:
                    iplot = 3
                if iplot > 0 and iplot < 3:
                    if event.button()==2:
                        flip = True
                        choose = True
                    elif event.button()==1:
                        if event.double():
                            zoom = True
                        else:
                            choose = True
                if iplot==3 and event.double():
                    zoom = True
                posy = int(posy)
                posx = int(posx)
                if zoom:
                    if iplot==1:
                        self.p1.setXRange(0,self.ops['Ly'])
                        self.p1.setYRange(0,self.ops['Lx'])
                    elif iplot==2:
                        self.p2.setXRange(0,self.ops['Ly'])
                        self.p2.setYRange(0,self.ops['Lx'])
                    else:
                        self.p3.setXRange(0,self.Fcell.shape[1])
                        self.p3.setYRange(self.fmin,self.fmax)
                    self.show()
                if choose:
                    ichosen = int(self.iROI[posx,posy])
                    if self.ichosen == ichosen:
                        choose = False
                    elif ichosen >= 0:
                        self.ichosen = ichosen
                if flip and self.iflip != self.ichosen:
                    flip = True
                    iscell = int(self.iscell[self.ichosen])
                    if 2-iscell == iplot:
                        self.iscell[self.ichosen] = ~self.iscell[self.ichosen]
                        np.save(self.basename+'/iscell.npy', self.iscell)
                    self.iflip = self.ichosen
                    fig.flip_cell(self)
                else:
                    flip = False
                if choose or flip:
                    t0=time.time()
                    M = fig.draw_masks(self)
                    self.plot_masks(M)
                    self.plot_trace()
                    self.show()
                    print(time.time()-t0)

    def run_suite2p(self):
        RW = gui.RunWindow(self)
        RW.show()

    def load_dialog(self):
        name = QtGui.QFileDialog.getOpenFileName(self, 'Open File')
        self.fname = name[0]
        self.load_proc()

    def load_proc(self):
        name = self.fname
        print(name)
        try:
            self.stat = np.load(name)
            self.stat = self.stat.item()
            ypix = self.stat[0]['ypix']
        except (ValueError, KeyError, OSError, RuntimeError, TypeError, NameError):
            print('ERROR: this is not a stat.npy file :( (needs stat[n]["ypix"]!)')
            self.stat = None
        if self.stat is not None:
            basename, fname = os.path.split(name)
            self.basename = basename
            goodfolder = True
            try:
                self.Fcell = np.load(basename + '/F.npy')
                self.Fneu = np.load(basename + '/Fneu.npy')
            except (ValueError, OSError, RuntimeError, TypeError, NameError):
                print('ERROR: there are no fluorescence traces in this folder (F.npy/Fneu.npy)')
                goodfolder = False
            try:
                self.Spks = np.load(basename + '/spks.npy')
            except (ValueError, OSError, RuntimeError, TypeError, NameError):
                print('there are no spike deconvolved traces in this folder (spks.npy)')
            try:
                self.iscell = np.load(basename + '/iscell.npy')
            except (ValueError, OSError, RuntimeError, TypeError, NameError):
                print('no manual labels found (iscell.npy)')
            try:
                self.ops = np.load(basename + '/ops.npy')
                self.ops = self.ops.item()
            except (ValueError, OSError, RuntimeError, TypeError, NameError):
                print('ERROR: there is no ops file in this folder (ops.npy)')
                goodfolder = False
            if goodfolder:
                self.make_masks_and_buttons(name[0])
                self.loaded = True
            else:
                print('stat.npy found, but other files not in folder')
                Text = 'stat.npy found, but other files missing, choose another?'
                self.load_again(Text)
        else:
            Text = 'Incorrect file, not a stat.npy, choose another?'
            self.load_again(Text)

    def load_again(self,Text):
        tryagain = QtGui.QMessageBox.question(self, 'ERROR',
                                        Text,
                                        QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        if tryagain == QtGui.QMessageBox.Yes:
            self.load_proc()

    def load_classifier(self):
        name = QtGui.QFileDialog.getOpenFileName(self, 'Open File')
        if name:
            self.model = classifier.Classifier(classfile=name[0],
                                               trainfiles=None,
                                               statclass=None)
            if self.model.loaded:
                # statistics from current dataset for Classifier
                checkstat = [key for key in self.statclass if key in self.stat[0]]
                if len(checkstat) == len(self.statclass):
                    ncells = self.Fcell.shape[0]
                    self.statistics = np.zeros((ncells, len(self.statclass)),np.float32)
                    k=0
                    for key in self.statclass:
                        for n in range(0,ncells):
                            self.statistics[n,k] = self.stat[n][key]
                        k+=1
                    self.trainfiles = self.model.trainfiles
                    self.activate_classifier()
                else:
                    print('ERROR: classifier has fields that stat doesn''t have')

    def load_traindata(self):
        # will return
        self.traindata = np.zeros((0,len(self.statclass)+1),np.float32)
        LC = gui.ListChooser('classifier training files', self)
        result = LC.exec_()
        if result:
            print('Populating classifier:')
            self.model = classifier.Classifier(classfile=None,
                                               trainfiles=self.trainfiles,
                                               statclass=self.statclass)
            if self.trainfiles is not None:
                ncells = self.Fcell.shape[0]
                self.statistics = np.zeros((ncells, len(self.statclass)),np.float32)
                k=0
                for key in self.statclass:
                    for n in range(0,ncells):
                        self.statistics[n,k] = self.stat[n][key]
                    k+=1
                self.activate_classifier()

    def apply_classifier(self):
        classval = self.probedit.value()
        self.iscell, self.probcell = self.model.apply(self.statistics, classval)

    def save_classifier(self):
        name = QtGui.QFileDialog.getSaveFileName(self,'Save classifier')
        if name:
            try:
                self.model.save_classifier(name[0])
            except (OSError, RuntimeError, TypeError, NameError,FileNotFoundError):
                print('ERROR: incorrect filename for saving')

    def save_trainlist(self):
        name = QtGui.QFileDialog.getSaveFileName(self,'Save list of iscell.npy')
        if name:
            try:
                with open(name[0],'w') as fid:
                    for f in self.trainfiles:
                        fid.write(f)
                        fid.write('\n')
            except (ValueError, OSError, RuntimeError, TypeError, NameError,FileNotFoundError):
                print('ERROR: incorrect filename for saving')

    def activate_classifier(self):
        iscell, self.probcell = self.model.apply(self.statistics, 0.5)
        istat = self.probcell
        if len(self.clabels) < self.ncolors:
            self.clabels.append([istat.min(), (istat.max()-istat.min())/2, istat.max()])
        else:
            self.clabels[-1] = [istat.min(), (istat.max()-istat.min())/2, istat.max()]
        istat = istat - istat.min()
        istat = istat / istat.max()
        istat = istat / 1.3
        istat = istat + 0.1
        icols = 1 - istat
        if self.ops_plot[3].shape[1] < self.ncolors:
            self.ops_plot[3] = np.concatenate((self.ops_plot[3],
                                                np.expand_dims(icols,axis=1)), axis=1)
        else:
            self.ops_plot[3][:,-1] = icols
        self.classbtn.setEnabled(True)
        self.saveClass.setEnabled(True)
        self.saveTrain.setEnabled(True)
        applyclass = QtGui.QPushButton('apply classifier')
        applyclass.resize(100,50)
        applyclass.clicked.connect(self.apply_classifier)
        self.l0.addWidget(QtGui.QLabel('\t      cell prob'),13,0,1,1)
        self.probedit = QtGui.QDoubleSpinBox(self)
        self.probedit.setDecimals(3)
        self.probedit.setMaximum(1.0)
        self.probedit.setMinimum(0.0)
        self.probedit.setSingleStep(0.01)
        self.probedit.setValue(0.5)
        #self.probedit.setValidator(QtGui.QDoubleValidator())
        #self.probedit.setText('0.5')
        #self.probedit.setMaxLength(5)
        #qedit.move(10,600)
        self.probedit.setFixedWidth(55)
        self.l0.addWidget(self.probedit,13,0,1,1)
        self.l0.addWidget(applyclass,14,0,1,1)
        addtoclass = QtGui.QPushButton('add current data \n to classifier')
        addtoclass.resize(100,100)
        addtoclass.clicked.connect(self.add_to_classifier)
        self.l0.addWidget(addtoclass,15,0,1,1)
        saveclass = QtGui.QPushButton('save classifier')
        saveclass.resize(100,50)
        saveclass.clicked.connect(self.save_trainlist)
        self.l0.addWidget(saveclass,16,0,1,1)

    def add_to_classifier(self):
        fname = self.basename+'/iscell.npy'
        ftrue =  [f for f in self.trainfiles if fname in f]
        if len(ftrue)==0:
            self.trainfiles.append(self.basename+'/iscell.npy')
        print('Repopulating classifier including current dataset:')
        self.model = classifier.Classifier(classfile=None,
                                           trainfiles=self.trainfiles,
                                           statclass=self.statclass)

def run():
    ## Always start by initializing Qt (only once per application)
    app = QtGui.QApplication(sys.argv)
    GUI = MainW()
    sys.exit(app.exec_())

run()



import numpy as np
import math, time
import glob, h5py, os
from scipy import signal
from suite2p import celldetect2 as celldetect2
from scipy import stats, io, signal

from skimage import io
def tic():
    return time.time()
def toc(i0):
    return time.time() - i0

def fitMVGaus(y,x,lam,thres=2.5):
    ''' computes 2D gaussian fit to data and returns ellipse of radius thres standard deviations
    inputs:
        y, x, lam, thres
            y,x: pixel locations
            lam: pixel weights
        thres: number of standard deviations at which to draw ellipse
    outputs:
        mu, cov, ellipse, area
            mu: mean of gaussian fit
            cov: covariance of gaussian fit
            radii: half of major and minor axis lengths of elliptical fit
            ellipse: coordinates of elliptical fit
            area: area of ellipse
    '''
    # normalize pixel weights
    lam = lam / lam.sum()
    # mean of gaussian
    yx = np.stack((y,x))
    mu  = (lam*yx).sum(axis=-1)
    yx = yx - np.expand_dims(mu, axis=1)
    yx = yx * lam**.5
    #yx  = np.concatenate((y*lam**0.5, x*lam**0.5),axis=0)
    cov = yx @ yx.transpose()
    # radii of major and minor axes
    radii,evec  = np.linalg.eig(cov)
    radii = np.maximum(0, np.real(radii))
    radii       = thres * radii**.5
    # compute pts of ellipse
    npts = 100
    p = np.expand_dims(np.linspace(0, 2*math.pi, npts),axis=1)
    p = np.concatenate((np.cos(p), np.sin(p)),axis=1)
    ellipse = (p * radii) @ evec.transpose() + mu
    area = (radii[0] * radii[1])**0.5 * math.pi
    radii  = np.sort(radii)[::-1]
    return mu, cov, radii, ellipse, area

def enhanced_mean_image(ops):
    if 1:
        I = ops['meanImg']
        Imed = signal.medfilt2d(I, 4*ops['diameter']+1)
        I = I - Imed
        Idiv = signal.medfilt2d(np.absolute(I), 4*ops['diameter']+1)
        I = I / (1e-10 + Idiv)
        mimg1 = -6
        mimg99 = 6
        mimg0 = I
    else:
        mimg0 = ops['meanImg']
        mimg0 = mimg0 - gaussian_filter(filters.minimum_filter(mimg0,50,mode='mirror'),
                                          10,mode='mirror')
        mimg0 = mimg0 / gaussian_filter(filters.maximum_filter(mimg0,50,mode='mirror'),
                                      10,mode='mirror')
        mimg1 = np.percentile(mimg0,1)
        mimg99 = np.percentile(mimg0,99)
    mimg0 = mimg0[ops['yrange'][0]:ops['yrange'][1], ops['xrange'][0]:ops['xrange'][1]]
    mimg0 = (mimg0 - mimg1) / (mimg99 - mimg1)
    mimg0 = np.maximum(0,np.minimum(1,mimg0))
    mimg = mimg0.min() * np.ones((ops['Ly'],ops['Lx']),np.float32)
    mimg[ops['yrange'][0]:ops['yrange'][1],
        ops['xrange'][0]:ops['xrange'][1]] = mimg0
    ops['meanImgE'] = mimg
    return ops

def h5py_to_binary(ops):
    nplanes = ops['nplanes']
    nchannels = ops['nchannels']
    ops1 = []
    # open all binary files for writing
    reg_file = []
    if nchannels>1:
        reg_file_chan2 = []
    for j in range(0,nplanes):
        ops['save_path'] = os.path.join(ops['save_path0'], 'suite2p', 'plane%d'%j)
        if ('fast_disk' not in ops) or len(ops['fast_disk'])>0:
            ops['fast_disk'] = ops['save_path0']
        ops['fast_disk'] = os.path.join(ops['fast_disk'], 'suite2p', 'plane%d'%j)
        ops['ops_path'] = os.path.join(ops['save_path'],'ops.npy')
        ops['reg_file'] = os.path.join(ops['fast_disk'], 'data.bin')
        if nchannels>1:
            ops['reg_file_chan2'] = os.path.join(ops['fast_disk'], 'data_chan2.bin')
        if not os.path.isdir(ops['fast_disk']):
            os.makedirs(ops['fast_disk'])
        if not os.path.isdir(ops['save_path']):
            os.makedirs(ops['save_path'])
        ops1.append(ops.copy())
        reg_file.append(open(ops['reg_file'], 'wb'))
        if nchannels>1:
            reg_file_chan2.append(open(ops['reg_file_chan2'], 'wb'))

    # open h5py file for reading
    key = ops['h5py_key']

    with h5py.File(ops['h5py'], 'r') as f:
        # keep track of the plane identity of the first frame (channel identity is assumed always 0)
        nbatch = nplanes*nchannels*math.ceil(ops['batch_size']/(nplanes*nchannels))
        nframes_all = f[key].shape[0]
        # loop over all tiffs
        i0 = 0
        while 1:
            irange = np.arange(i0, min(i0+nbatch, nframes_all), 1)
            if irange.size==0:
                break
            im = f[key][irange, :, :]
            nframes = im.shape[0]
            for j in range(0,nplanes):
                im2write = im[np.arange(j, nframes, nplanes*nchannels),:,:]
                reg_file[j].write(bytearray(im2write))
                if nchannels>1:
                    im2write = im[np.arange(j+1, nframes, nplanes*nchannels),:,:]
                    reg_file_chan2[j].write(bytearray(im2write))
            i0 += nframes
    # write ops files
    for ops in ops1:
        ops['Ly'] = im2write.shape[1]
        ops['Lx'] = im2write.shape[2]
        np.save(ops['ops_path'], ops)
    # close all binary files and write ops files
    for j in range(0,nplanes):
        reg_file[j].close()
        if nchannels>1:
            reg_file_chan2[j].close()
    return ops1

def tiff_to_binary(ops):
    nplanes = ops['nplanes']
    nchannels = ops['nchannels']
    ops1 = []
    # open all binary files for writing
    reg_file = []
    if nchannels>1:
        reg_file_chan2 = []
    for j in range(0,nplanes):
        fpath = os.path.join(ops['save_path0'], 'suite2p', 'plane%d'%j)
        ops['save_path'] = fpath
        if ('fast_disk' not in ops) or len(ops['fast_disk'])>0:
            ops['fast_disk'] = ops['save_path0']
        ops['fast_disk'] = os.path.join(ops['fast_disk'], 'suite2p', 'plane%d'%j)
        ops['ops_path'] = os.path.join(ops['save_path'],'ops.npy')
        ops['reg_file'] = os.path.join(ops['fast_disk'], 'data.bin')
        if nchannels>1:
            ops['reg_file_chan2'] = os.path.join(ops['fast_disk'], 'data_chan2.bin')
        if not os.path.isdir(ops['fast_disk']):
            os.makedirs(ops['fast_disk'])
        if not os.path.isdir(ops['save_path']):
            os.makedirs(ops['save_path'])
        ops1.append(ops.copy())
        reg_file.append(open(ops['reg_file'], 'wb'))
        if nchannels>1:
            reg_file_chan2.append(open(ops['reg_file_chan2'], 'wb'))
    fs, ops = get_tif_list(ops) # look for tiffs in all requested folders
    # loop over all tiffs
    for ik, file in enumerate(fs):
        # keep track of the plane identity of the first frame (channel identity is assumed always 0)
        if ops['first_tiffs'][ik]:
            iplane = 0
        im = io.imread(file)
        if len(im.shape)<3:
            im = np.expand_dims(im, axis=0)
        nframes = im.shape[0]
        for j in range(0,nplanes):
            i0 = nchannels * ((iplane+j)%nplanes)
            im2write = im[np.arange(int(i0), nframes, nplanes*nchannels),:,:]
            reg_file[j].write(bytearray(im2write))
            if nchannels>1:
                im2write = im[np.arange(int(i0)+1, nframes, nplanes*nchannels),:,:]
                reg_file_chan2[j].write(bytearray(im2write))
        iplane = (iplane+nframes/nchannels)%nplanes
    # write ops files
    for ops in ops1:
        ops['Ly'] = im.shape[1]
        ops['Lx'] = im.shape[2]
        np.save(ops['ops_path'], ops)
    # close all binary files and write ops files
    for j in range(0,nplanes):
        reg_file[j].close()
        if nchannels>1:
            reg_file_chan2[j].close()
    return ops1

def list_tifs(froot, look_one_level_down):
    lpath = os.path.join(froot, "*.tif")
    fs  = sorted(glob.glob(lpath))
    lpath = os.path.join(froot, "*.tiff")
    fs2 = sorted(glob.glob(lpath))
    fs.extend(fs2)
    if look_one_level_down:
        fdir = glob.glob(os.path.join(froot, "*", ""))
        for folder_down in fdir:
            lpath = os.path.join(froot, folder_down, "*.tif")
            fs3 = sorted(glob.glob(lpath))
            lpath = os.path.join(froot, folder_down, "*.tiff")
            fs4 = sorted(glob.glob(lpath))
            fs.extend(fs3)
            fs.extend(fs4)
    return fs

def get_tif_list(ops):
    froot = ops['data_path']
    if len(froot)==1:
        if len(ops['subfolders'])==0:
            fold_list = ops['data_path']
        else:
            fold_list = []
            for folder_down in ops['subfolders']:
                fold = os.path.join(froot[0], folder_down)
                fold_list.append(fold)
    else:
        fold_list = froot
    fs = []
    nfs = 0
    ix = np.zeros((len(fold_list), 1), 'int32')
    for k,fld in enumerate(fold_list):
        ix[k] = len(fs)
        fs.extend(list_tifs(fld, ops['look_one_level_down']))
    ops['first_tiffs'] = np.zeros(len(fs), 'bool_')
    ops['first_tiffs'][ix] = True
    if len(fs)==0:
        raise Exception('Could not find any tifs')
    else:
        print('Found %d tifs'%(len(fs)))
    return fs, ops

def get_tif_list_old(ops):
    froot = ops['data_path']
    if len(ops['subfolders'])==0:
        fs = list_tifs(ops, froot)
    else:
        fs = []
        for folder_down in ops['subfolders']:
            fold = os.path.join(froot, folder_down)
            fs.extend(list_tifs(ops, fold))
    if fs is None:
        raise Exception('Could not find any tifs')


def get_cells(ops):
    i0 = tic()
    if (type(ops['diameter']) is int) or len(ops['diameter'])<2:
        ops['diameter'] = int(np.array(ops['diameter']))
        ops['diameter'] = np.array((ops['diameter'], ops['diameter']))
    ops['diameter'] = np.array(ops['diameter']).astype('int32')
    print(ops['diameter'])
    ops, stat = celldetect2.sourcery(ops)
    print('time %4.4f. Found %d ROIs'%(toc(i0), len(stat)))
    # extract fluorescence and neuropil
    F, Fneu, ops = celldetect2.extractF(ops, stat)
    print('time %4.4f. Extracted fluorescence from %d ROIs'%(toc(i0), len(stat)))
    # subtract neuropil
    dF = F - ops['neucoeff'] * Fneu
    # compute activity statistics for classifier
    sk = stats.skew(dF, axis=1)
    sd = np.std(dF, axis=1)
    npix = np.array([stat[n]['npix'] for n in range(len(stat))]).astype('float32')
    npix /= np.mean(npix)
    for k in range(F.shape[0]):
        stat[k]['skew'] = sk[k]
        stat[k]['std']  = sd[k]
        stat[k]['npix_norm'] = npix[k]
    # add enhanced mean image
    ops = enhanced_mean_image(ops)
    # save ops
    np.save(ops['ops_path'], ops)
    # save results
    fpath = ops['save_path']
    np.save(os.path.join(fpath,'F.npy'), F)
    np.save(os.path.join(fpath,'Fneu.npy'), Fneu)
    np.save(os.path.join(fpath,'stat.npy'), stat)
    iscell = np.ones((len(stat),2))
    np.save(os.path.join(fpath, 'iscell.npy'), iscell)
    print('results saved to %s'%ops['save_path'])
    return ops

def combined(ops1):
    '''
    Combines all the entries in ops1 into a single result file.
    Multi-plane recordings are arranged to best tile a square.
    Multi-roi recordings are arranged by their dx,dy physical localization.
    '''
    ops = ops1[0]
    if ('dx' not in ops) or ('dy' not in ops):
        Lx = ops['Lx']
        Ly = ops['Ly']
        nX = np.ceil(np.sqrt(ops['Ly'] * ops['Lx'] * len(ops1))/ops['Lx'])
        nX = int(nX)
        nY = int(np.ceil(len(ops1)/nX))
        for j in range(len(ops1)):
            ops1[j]['dx'] = (j%nX) * Lx
            ops1[j]['dy'] = int(j/nX) * Ly
    LY = int(np.amax(np.array([ops['Ly']+ops['dy'] for ops in ops1])))
    LX = int(np.amax(np.array([ops['Lx']+ops['dx'] for ops in ops1])))
    meanImg = np.zeros((LY, LX))
    meanImgE = np.zeros((LY, LX))
    Vcorr = np.zeros((LY, LX))
    Nfr = np.amax(np.array([ops['nframes'] for ops in ops1]))
    for k,ops in enumerate(ops1):
        fpath = ops['save_path']
        stat0 = np.load(os.path.join(fpath,'stat.npy'))
        xrange = np.arange(ops['dx'],ops['dx']+ops['Lx'])
        yrange = np.arange(ops['dy'],ops['dy']+ops['Ly'])
        meanImg[np.ix_(yrange, xrange)] = ops['meanImg']
        meanImgE[np.ix_(yrange, xrange)] = ops['meanImgE']
        xrange = np.arange(ops['dx']+ops['xrange'][0],ops['dx']+ops['xrange'][-1])
        yrange = np.arange(ops['dy']+ops['yrange'][0],ops['dy']+ops['yrange'][-1])
        Vcorr[np.ix_(yrange, xrange)] = ops['Vcorr']
        for j in range(len(stat0)):
            stat0[j]['xpix'] += ops['dx']
            stat0[j]['ypix'] += ops['dy']
            stat0[j]['med'][0] += ops['dy']
            stat0[j]['med'][1] += ops['dx']
        F0    = np.load(os.path.join(fpath,'F.npy'))
        Fneu0 = np.load(os.path.join(fpath,'Fneu.npy'))
        spks0 = np.load(os.path.join(fpath,'spks.npy'))
        iscell0 = np.load(os.path.join(fpath,'iscell.npy'))
        nn,nt = F0.shape
        if nt<Nfr:
            fcat    = np.zeros((nn,Nfr-nt), 'float32')
            print(F0.shape)
            print(fcat.shape)
            F0      = np.concatenate((F0, fcat), axis=1)
            spks0   = np.concatenate((spks0, fcat), axis=1)
            Fneu0   = np.concatenate((Fneu0, fcat), axis=1)
        if k==0:
            F, Fneu, spks,stat,iscell = F0, Fneu0, spks0,stat0, iscell0
        else:
            F    = np.concatenate((F, F0))
            Fneu = np.concatenate((Fneu, Fneu0))
            spks = np.concatenate((spks, spks0))
            stat = np.concatenate((stat,stat0))
            iscell = np.concatenate((iscell,iscell0))
    ops['meanImg']  = meanImg
    ops['meanImgE'] = meanImgE
    ops['Vcorr'] = Vcorr
    ops['Ly'] = LY
    ops['Lx'] = LX
    ops['xrange'] = [0, ops['Lx']]
    ops['yrange'] = [0, ops['Ly']]
    fpath = os.path.join(ops['save_path0'], 'suite2p', 'combined')
    if not os.path.isdir(fpath):
        os.makedirs(fpath)
    ops['save_path'] = fpath
    np.save(os.path.join(fpath, 'F.npy'), F)
    np.save(os.path.join(fpath, 'Fneu.npy'), Fneu)
    np.save(os.path.join(fpath, 'spks.npy'), spks)
    np.save(os.path.join(fpath, 'ops.npy'), ops)
    np.save(os.path.join(fpath, 'stat.npy'), stat)
    np.save(os.path.join(fpath, 'iscell.npy'), iscell)

    # save as matlab file
    if ('save_mat' in ops) and ops['save_mat']:
        matpath = os.path.join(ops['save_path'],'Fall.mat')
        scipy.io.savemat(matpath, {'stat': stat,
                                   'ops': ops,
                                   'F': F,
                                   'Fneu': Fneu,
                                   'spks': spks,
                                   'iscell': iscell})
    return ops



from PyQt5 import QtGui, QtCore
import pyqtgraph as pg
from pyqtgraph import console
import sys
import numpy as np
import os
import pickle
from suite2p import fig
from suite2p import run_s2p

# this file contains helper functions for GUI and run dialog

### custom QDialog which allows user to fill in ops and run suite2p!
class RunWindow(QtGui.QDialog):
    def __init__(self, parent=None):
        super(RunWindow, self).__init__(parent)
        self.setGeometry(50,50,1200,800)
        self.setWindowTitle('Choose run options')
        self.win = QtGui.QWidget(self)
        self.layout = QtGui.QGridLayout()
        self.layout.setHorizontalSpacing(25)
        self.win.setLayout(self.layout)
        # initial ops values
        self.ops = run_s2p.default_ops()
        self.data_path = []
        self.save_path = []
        self.fast_disk = []
        tifkeys = ['nplanes','nchannels','functional_chan','diameter','tau','fs']
        parkeys = ['num_workers','num_workers_roi','combined']
        regkeys = ['nimg_init', 'batch_size', 'maxregshift', 'align_by_chan', 'reg_tif']
        cellkeys = ['max_overlap','threshold_scaling','max_iterations','navg_frames_svd','nsvd_for_roi','tile_factor']
        neukeys = ['ratio_neuropil_to_cell','inner_neuropil_radius','outer_neuropil_radius','min_neuropil_pixels']
        deconvkeys = ['win_baseline','sig_baseline','prctile_baseline','neucoeff']
        keys = [[],tifkeys, parkeys, regkeys, cellkeys, neukeys, deconvkeys]
        labels = ['Filepaths','Main settings','Parallel','Registration','ROI detection','Neuropil','Deconvolution']
        tooltips = ['each tiff has this many planes in sequence',
                    'each tiff has this many channels per plane',
                    'this channel is used to extract functional ROIs (1-based)',
                    'approximate diameter of ROIs in pixels (can input two numbers separated by a comma for elongated ROIs)',
                    'timescale of sensor in deconvolution (in seconds)',
                    'sampling rate (per plane)',
                    '0 to select num_cores, -1 to disable parallelism, N to enforce value',
                    'ROI detection parallelism: 0 to select number of planes, -1 to disable parallelism, N to enforce value',
                    'combine results across planes in separate folder "combined" at end of processing',
                    '# of subsampled frames for finding reference image',
                    'number of frames per batch',
                    'max allowed registration shift, as a fraction of frame max(width and height)',
                    'when multi-channel, you can align by non-functional channel (1-based)',
                    'if 1, registered tiffs are saved',
                    'ROIs with greater than this overlap as a fraction of total pixels will be discarded',
                    'adjust the automatically determined threshold by this scalar multiplier',
                    'maximum number of iterations for ROI detection',
                    'max number of binned frames for the SVD',
                    'max number of SVD components to keep for ROI detection',
                    'tile factor',
                    'minimum ratio between neuropil radius and cell radius',
                    'number of pixels between ROI and neuropil donut',
                    'maximum neuropil radius',
                    'minimum number of pixels in the neuropil',
                    'window for maximin',
                    'smoothing constant for gaussian filter',
                    'smoothing constant for gaussian filter',
                    'neuropil coefficient']
        l=0
        self.keylist = []
        self.editlist = []
        kk=0
        for lkey in keys:
            k = 0
            qlabel = QtGui.QLabel(labels[l])
            bigfont = QtGui.QFont("Arial", 10, QtGui.QFont.Bold)
            qlabel.setFont(bigfont)
            self.layout.addWidget(qlabel,0,2*l,1,2)
            for key in lkey:
                lops = 1
                if self.ops[key] or (self.ops[key] == 0):
                    qedit = QtGui.QLineEdit()
                    qlabel = QtGui.QLabel(key)
                    qlabel.setToolTip(tooltips[kk])
                    if key == 'diameter':
                        if (type(self.ops[key]) is not int) and (len(self.ops[key])>1):
                            dstr = str(int(self.ops[key][0])) + ', ' + str(int(self.ops[key][1]))
                        else:
                            dstr = str(int(self.ops[key]))
                    else:
                        if type(self.ops[key]) is not bool:
                            dstr = str(self.ops[key])
                        else:
                            dstr = str(int(self.ops[key]))
                    qedit.setText(dstr)
                    qedit.setFixedWidth(105)
                    self.layout.addWidget(qlabel,k*2+1,2*l,1,2)
                    self.layout.addWidget(qedit,k*2+2,2*l,1,2)
                    self.keylist.append(key)
                    self.editlist.append(qedit)
                k+=1
                kk+=1
            l+=1
        # data_path
        key = 'look_one_level_down'
        qlabel = QtGui.QLabel(key)
        qlabel.setToolTip('whether to look in all subfolders when searching for tiffs')
        self.layout.addWidget(qlabel,1,0,1,1)
        qedit = QtGui.QLineEdit()
        self.layout.addWidget(qedit,2,0,1,1)
        qedit.setText(str(int(self.ops[key])))
        qedit.setFixedWidth(105)
        self.keylist.append(key)
        self.editlist.append(qedit)
        btiff = QtGui.QPushButton('Add directory to data_path')
        btiff.clicked.connect(self.get_folders)
        self.layout.addWidget(btiff,3,0,1,1)
        qlabel = QtGui.QLabel('data_path')
        qlabel.setFont(bigfont)
        self.layout.addWidget(qlabel,4,0,1,1)
        # save_path0
        bsave = QtGui.QPushButton('Add save_path (default is 1st data_path)')
        bsave.clicked.connect(self.save_folder)
        self.layout.addWidget(bsave,15,0,1,1)
        self.savelabel = QtGui.QLabel('')
        self.layout.addWidget(self.savelabel,16,0,1,1)
        # fast_disk
        bbin = QtGui.QPushButton('Add fast_disk (default is save_path)')
        bbin.clicked.connect(self.bin_folder)
        self.layout.addWidget(bbin,17,0,1,1)
        self.binlabel = QtGui.QLabel('')
        self.layout.addWidget(self.binlabel,18,0,1,1)
        self.runButton = QtGui.QPushButton('RUN SUITE2P')
        self.runButton.clicked.connect(lambda: self.run_S2P(parent))
        self.layout.addWidget(self.runButton,20,0,1,1)
        self.textEdit = QtGui.QTextEdit()
        self.layout.addWidget(self.textEdit, 21,0,30,2*l)
        self.process = QtCore.QProcess(self)
        self.process.readyReadStandardOutput.connect(self.stdout_write)
        self.process.readyReadStandardError.connect(self.stderr_write)
        # disable the button when running the s2p process
        self.process.started.connect(self.started)
        self.process.finished.connect(lambda: self.finished(parent))
        # stop process
        self.stopButton = QtGui.QPushButton('STOP')
        self.stopButton.setEnabled(False)
        self.layout.addWidget(self.stopButton, 20,2,1,2)
        self.stopButton.clicked.connect(self.stop)

    def stop(self):
        self.finish = False
        self.process.kill()

    def started(self):
        self.runButton.setEnabled(False)
        self.stopButton.setEnabled(True)

    def finished(self, parent):
        self.runButton.setEnabled(True)
        self.stopButton.setEnabled(False)
        if self.finish and not self.error:
            cursor = self.textEdit.textCursor()
            cursor.movePosition(cursor.End)
            cursor.insertText('Opening in GUI (can close this window)\n')
            parent.fname = os.path.join(self.save_path, 'suite2p', 'plane0','stat.npy')
            parent.load_proc()
        elif not self.error:
            cursor = self.textEdit.textCursor()
            cursor.movePosition(cursor.End)
            cursor.insertText('Interrupted by user (not finished)\n')
        else:
            cursor = self.textEdit.textCursor()
            cursor.movePosition(cursor.End)
            cursor.insertText('Interrupted by error (not finished)\n')

    def stdout_write(self):
        cursor = self.textEdit.textCursor()
        cursor.movePosition(cursor.End)
        cursor.insertText(str(self.process.readAllStandardOutput(), 'utf-8'))
        self.textEdit.ensureCursorVisible()

    def stderr_write(self):
        cursor = self.textEdit.textCursor()
        cursor.movePosition(cursor.End)
        cursor.insertText('>>>ERROR<<<\n')
        cursor.insertText(str(self.process.readAllStandardError(), 'utf-8'))
        self.textEdit.ensureCursorVisible()
        self.error = True

    def run_S2P(self, parent):
        self.finish = True
        self.error = False
        k=0
        for key in self.keylist:
            if key=='diameter':
                diams = self.editlist[k].text().replace(' ','').split(',')
                if len(diams)>1:
                    self.ops[key] = [int(diams[0]), int(diams[1])]
                else:
                    self.ops[key] = int(diams[0])
            else:
                if type(self.ops[key]) is float:
                    self.ops[key] = float(self.editlist[k].text())
                    #print(key,'\t\t', float(self.editlist[k].text()))
                elif type(self.ops[key]) is int or bool:
                    self.ops[key] = int(self.editlist[k].text())
                    #print(key,'\t\t', int(self.editlist[k].text()))
            k+=1
        self.db = {}
        self.db['data_path'] = self.data_path
        self.db['subfolders'] = []
        if len(self.save_path)==0:
            fpath = self.db['data_path'][0]
            self.save_path = fpath
        self.db['save_path0'] = self.save_path
        if len(self.fast_disk)==0:
            self.fast_disk = self.save_path
        self.db['fast_disk'] = self.fast_disk
        print('Running suite2p!')
        print('starting process')
        np.save('ops.npy', self.ops)
        np.save('db.npy', self.db)
        self.process.start('python -u -W ignore -m suite2p --ops ops.npy --db db.npy')

    def get_folders(self):
        name = QtGui.QFileDialog.getExistingDirectory(self, "Add directory to data path")
        self.data_path.append(name)
        self.layout.addWidget(QtGui.QLabel(name),
                              len(self.data_path)+4,0,1,1)

    def save_folder(self):
        name = QtGui.QFileDialog.getExistingDirectory(self, "Save folder for data")
        self.save_path = name
        self.savelabel.setText(name)

    def bin_folder(self):
        name = QtGui.QFileDialog.getExistingDirectory(self, "Folder for binary file")
        self.fast_disk = name
        self.binlabel.setText(name)

# custom vertical label
class VerticalLabel(QtGui.QWidget):
    def __init__(self, text=None):
        super(self.__class__, self).__init__()
        self.text = text

    def paintEvent(self, event):
        painter = QtGui.QPainter(self)
        painter.setPen(QtCore.Qt.white)
        painter.translate(0, 0)
        painter.rotate(90)
        if self.text:
            painter.drawText(0, 0, self.text)
        painter.end()

### custom QPushButton class that plots image when clicked
# requires buttons to put into a QButtonGroup (parent.viewbtns)
# allows only 1 button to pressed at a time
class ViewButton(QtGui.QPushButton):
    def __init__(self, bid, Text, parent=None):
        super(ViewButton,self).__init__(parent)
        self.setText(Text)
        self.setCheckable(True)
        self.setStyleSheet(parent.styleInactive)
        self.resize(self.minimumSizeHint())
        self.clicked.connect(lambda: self.press(parent, bid))
        self.show()
    def press(self, parent, bid):
        for b in parent.viewbtns.buttons():
            b.setStyleSheet(parent.styleUnpressed)
        self.setStyleSheet(parent.stylePressed)
        parent.ops_plot[1] = bid
        if parent.ops_plot[2] == parent.ops_plot[3].shape[1]:
            fig.draw_corr(parent)
        M = fig.draw_masks(parent)
        fig.plot_masks(parent,M)

### Changes colors of ROIs
# button group is exclusive (at least one color is always chosen)
class ColorButton(QtGui.QPushButton):
    def __init__(self, bid, Text, parent=None):
        super(ColorButton,self).__init__(parent)
        self.setText(Text)
        self.setCheckable(True)
        self.setStyleSheet(parent.styleInactive)
        self.resize(self.minimumSizeHint())
        self.clicked.connect(lambda: self.press(parent, bid))
        self.show()
    def press(self, parent, bid):
        for b in parent.colorbtns.buttons():
            b.setStyleSheet(parent.styleUnpressed)
        self.setStyleSheet(parent.stylePressed)
        parent.ops_plot[2] = bid
        # disable top click
        if bid==0:
            for b in [1,2,4,5]:
                parent.topbtns.button(b).setStyleSheet(parent.styleInactive)
                parent.topbtns.button(b).setEnabled(False)
        else:
            for b in [1,2,4,5]:
                parent.topbtns.button(b).setStyleSheet(parent.styleUnpressed)
                parent.topbtns.button(b).setEnabled(True)
        if bid==6:
            fig.corr_masks(parent)
        M = fig.draw_masks(parent)
        fig.plot_masks(parent,M)
        fig.plot_colorbar(parent,bid)

# size of view
class SizeButton(QtGui.QPushButton):
    def __init__(self, bid, Text, parent=None):
        super(SizeButton,self).__init__(parent)
        self.setText(Text)
        self.setCheckable(True)
        self.setStyleSheet(parent.styleInactive)
        self.resize(self.minimumSizeHint())
        self.clicked.connect(lambda: self.press(parent, bid))
        self.show()
    def press(self, parent, bid):
        for b in parent.sizebtns.buttons():
            b.setStyleSheet(parent.styleUnpressed)
        self.setStyleSheet(parent.stylePressed)
        ts = 100
        if bid==0:
            parent.p2.linkView(parent.p2.XAxis,view=None)
            parent.p2.linkView(parent.p2.YAxis,view=None)
            parent.win.ci.layout.setColumnStretchFactor(0,ts)
            parent.win.ci.layout.setColumnStretchFactor(1,0)
        elif bid==1:
            parent.win.ci.layout.setColumnStretchFactor(0,ts)
            parent.win.ci.layout.setColumnStretchFactor(1,ts)
            parent.p2.setXLink('plot1')
            parent.p2.setYLink('plot1')
        elif bid==2:
            parent.p2.linkView(parent.p2.XAxis,view=None)
            parent.p2.linkView(parent.p2.YAxis,view=None)
            parent.win.ci.layout.setColumnStretchFactor(0,0)
            parent.win.ci.layout.setColumnStretchFactor(1,ts)
        parent.zoom_plot(1)
        parent.win.show()
        parent.show()
        parent.zoom_plot(1)
        parent.win.show()
        parent.show()

# selection of top neurons
class TopButton(QtGui.QPushButton):
    def __init__(self, bid, parent=None):
        super(TopButton,self).__init__(parent)
        text = [' draw selection', ' select top', ' select bottom',
                ' draw selection', ' select top', ' select bottom']
        self.setText(text[bid])
        self.setCheckable(True)
        self.setStyleSheet(parent.styleInactive)
        self.resize(self.minimumSizeHint())
        self.clicked.connect(lambda: self.press(parent, bid))
        self.show()
    def press(self, parent, bid):
        for b in parent.topbtns.buttons():
            b.setStyleSheet(parent.styleUnpressed)
        self.setStyleSheet(parent.stylePressed)
        if bid==0 or bid==3:
            parent.ROI_selection(bid)
        else:
            parent.top_selection(bid)



from PyQt5 import QtGui, QtCore
import sys
import numpy as np
from scipy.ndimage import gaussian_filter
import os
from sklearn.linear_model  import LogisticRegression
from suite2p import fig
import shutil

class Classifier:
    def __init__(self, classfile=None):
        # stat are cell stats from currently loaded recording
        # classfile is a previously saved classifier file
        if classfile is not None:
            self.classfile = classfile
            self.load()
        else:
            self.loaded = False

    def load(self):
        try:
            model = np.load(self.classfile)
            model = model.item()
            self.stats = model['stats']
            self.iscell = model['iscell']
            self.keys = model['keys']
            self.loaded = True
        except (ValueError, KeyError, OSError, RuntimeError, TypeError, NameError):
            print('ERROR: incorrect classifier file')
            self.loaded = False

    def apply(self, stat):
        y_pred     = probability(stat,self.stats,self.iscell,self.keys)
        return y_pred

def get_logp(test_stats, grid, p):
    nroi, nstats = test_stats.shape
    logp = np.zeros((nroi,nstats))
    for n in range(nstats):
        x = test_stats[:,n]
        x[x<grid[0,n]]   = grid[0,n]
        x[x>grid[-1,n]]  = grid[-1,n]
        ibin = np.digitize(x, grid[:,n], right=True) - 1
        logp[:,n] = np.log(p[ibin,n] + 1e-6) - np.log(1-p[ibin,n] + 1e-6)
    return logp

def probability(stat, train_stats, train_iscell, keys):
    nodes = 100
    nroi, nstats = train_stats.shape
    ssort= np.sort(train_stats, axis=0)
    isort= np.argsort(train_stats, axis=0)
    ix = np.linspace(0, nroi-1, nodes).astype('int32')
    grid = ssort[ix, :]
    p = np.zeros((nodes-1,nstats))
    for j in range(nodes-1):
        for k in range(nstats):
            p[j, k] = np.mean(train_iscell[isort[ix[j]:ix[j+1], k]])
    p = gaussian_filter(p, (2., 0))
    logp = get_logp(train_stats, grid, p)
    logisticRegr = LogisticRegression(C = 100.)
    logisticRegr.fit(logp, train_iscell)
    # now get logP from the test data
    test_stats = get_stat_keys(stat, keys)
    logp = get_logp(test_stats, grid, p)
    y_pred = logisticRegr.predict_proba(logp)
    y_pred = y_pred[:,1]
    return y_pred

def get_stat_keys(stat, keys):
    test_stats = np.zeros((len(stat), len(keys)))
    for j in range(len(stat)):
        for k in range(len(keys)):
            test_stats[j,k] = stat[j][keys[k]]
    return test_stats

def run(classfile,stat):
    model = Classifier(classfile=classfile)
    # compute cell probability
    probcell = model.apply(stat)
    iscell = probcell > 0.5
    iscell = np.concatenate((np.expand_dims(iscell,axis=1),np.expand_dims(probcell,axis=1)),axis=1)
    return iscell

def load(parent, name):
    print('loading classifier ', name)
    parent.model = Classifier(classfile=name)
    if parent.model.loaded:
        activate(parent, True)

def load_list(parent):
    # will return
    LC = ListChooser('classifier training files', parent)
    result = LC.exec_()

def load_data(parent,keys,trainfiles):
    train_stats = np.zeros((0,len(keys)),np.float32)
    train_iscell = np.zeros((0,),np.float32)
    trainfiles_good = []
    loaded = False
    if trainfiles is not None:
        for fname in trainfiles:
            badfile = False
            basename, bname = os.path.split(fname)
            try:
                iscells = np.load(fname)
                ncells = iscells.shape[0]
            except (ValueError, OSError, RuntimeError, TypeError, NameError):
                print('\t'+fname+': not a numpy array of booleans')
                badfile = True
            if not badfile:
                basename, bname = os.path.split(fname)
                lstat = 0
                try:
                    stat = np.load(basename+'/stat.npy')
                    ypix = stat[0]['ypix']
                    lstat = len(stat)
                except (KeyError, OSError, RuntimeError, TypeError, NameError):
                    print('\t'+basename+': incorrect or missing stat.npy file :(')
                if lstat != ncells:
                    print('\t'+basename+': stat.npy is not the same length as iscell.npy')
                else:
                    # add iscell and stat to classifier
                    print('\t'+fname+' was added to classifier')
                    iscell = iscells[:,0].astype(np.float32)
                    stats = get_stat_keys(stat,parent.default_keys)
                    train_stats = np.concatenate((train_stats,stats),axis=0)
                    train_iscell = np.concatenate((train_iscell,iscell),axis=0)
                    trainfiles_good.append(fname)
    if len(trainfiles_good) > 0:
        classfile, saved = save(parent,train_stats,train_iscell,keys)
        if saved:
            parent.classfile = classfile
            loaded = True
        else:
            msg = QtGui.QMessageBox.information(parent,'Incorrect file path',
                                                'Incorrect save path for classifier, classifier not built.')
    else:
        msg = QtGui.QMessageBox.information(parent,'Incorrect files',
                                            'No valid datasets chosen to build classifier, classifier not built.')
    return loaded

def add_to(parent):
    fname = parent.basename+'/iscell.npy'
    print('Adding current dataset to classifier')
    if parent.classfile == os.path.join(os.path.abspath(os.path.dirname(__file__)),
                     'classifiers/classifier_user.npy'):
        cfile = 'the default classifier'
    else:
        cfile = parent.classfile
    dm = QtGui.QMessageBox.question(parent,'Default classifier',
                                    'Current classifier is '+cfile+'. Add to this classifier?',
                                    QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
    if dm == QtGui.QMessageBox.Yes:
        stats = get_stat_keys(parent.stat, parent.model.keys)
        parent.model.stats = np.concatenate((parent.model.stats,stats),axis=0)
        parent.model.iscell = np.concatenate((parent.model.iscell,parent.iscell),axis=0)
        save_model(parent.classfile, parent.model.stats, parent.model.stats, parent.model.keys)
        activate(parent, True)
        msg = QtGui.QMessageBox.information(parent,'Classifier saved and loaded',
                                            'Current dataset added to classifier, and cell probabilities computed and in GUI')

def apply(parent):
    classval = parent.probedit.value()
    iscell = parent.probcell > classval
    fig.flip_for_class(parent, iscell)
    M = fig.draw_masks(parent)
    fig.plot_masks(parent,M)
    np.save(parent.basename+'/iscell.npy',
            np.concatenate((np.expand_dims(parent.iscell,axis=1),
            np.expand_dims(parent.probcell,axis=1)), axis=1))
    parent.lcell0.setText(' %d'%parent.iscell.sum())
    parent.lcell1.setText(' %d'%(parent.iscell.size-parent.iscell.sum()))

def save_model(name, train_stats, train_iscell, keys):
    model = {}
    model['stats']  = train_stats
    model['iscell'] = train_iscell
    model['keys']   = keys
    print('saving classifier in ' + name)
    np.save(name, model)

def save(parent, train_stats, train_iscell, keys):
    name = QtGui.QFileDialog.getSaveFileName(parent,'Classifier name (*.npy)')
    name = name[0]
    saved = False
    if name:
        try:
            save_model(name, train_stats, train_iscell, keys)
            saved = True
        except (OSError, RuntimeError, TypeError, NameError,FileNotFoundError):
            print('ERROR: incorrect filename for saving')
    return name, saved

def save_list(parent):
    name = QtGui.QFileDialog.getSaveFileName(parent,'Save list of iscell.npy')
    if name:
        try:
            with open(name[0],'w') as fid:
                for f in parent.trainfiles:
                    fid.write(f)
                    fid.write('\n')
        except (ValueError, OSError, RuntimeError, TypeError, NameError,FileNotFoundError):
            print('ERROR: incorrect filename for saving')

def activate(parent, inactive):
    if inactive:
        parent.probcell = parent.model.apply(parent.stat)
    istat = parent.probcell
    parent.clabels[-2] = [istat.min(), (istat.max()-istat.min())/2, istat.max()]
    istat = istat - istat.min()
    istat = istat / istat.max()
    istat = istat / 1.3
    istat = istat + 0.1
    icols = 1 - istat
    parent.ops_plot[3][:,-1] = icols
    fig.class_masks(parent)
    M = fig.draw_masks(parent)
    fig.plot_masks(parent,M)

def disable(parent):
    parent.classbtn.setEnabled(False)
    parent.saveClass.setEnabled(False)
    parent.saveTrain.setEnabled(False)
    for btns in parent.classbtns.buttons():
        btns.setEnabled(False)

### custom QDialog which makes a list of items you can include/exclude
class ListChooser(QtGui.QDialog):
    def __init__(self, Text, parent=None):
        super(ListChooser, self).__init__(parent)
        self.setGeometry(300,300,500,320)
        self.setWindowTitle(Text)
        self.win = QtGui.QWidget(self)
        layout = QtGui.QGridLayout()
        self.win.setLayout(layout)
        #self.setCentralWidget(self.win)
        loadcell = QtGui.QPushButton('Load iscell.npy')
        loadcell.resize(200,50)
        loadcell.clicked.connect(self.load_cell)
        layout.addWidget(loadcell,0,0,1,1)
        loadtext = QtGui.QPushButton('Load txt file list')
        loadtext.clicked.connect(self.load_text)
        layout.addWidget(loadtext,0,1,1,1)
        layout.addWidget(QtGui.QLabel('(select multiple using ctrl)'),1,0,1,1)
        self.list = QtGui.QListWidget(parent)
        layout.addWidget(self.list,2,0,5,4)
        #self.list.resize(450,250)
        self.list.setSelectionMode(QtGui.QAbstractItemView.MultiSelection)
        save = QtGui.QPushButton('build classifier')
        save.clicked.connect(lambda: self.build_classifier(parent))
        layout.addWidget(save,8,0,1,1)
        self.apply = QtGui.QPushButton('load in GUI')
        self.apply.clicked.connect(lambda: self.apply_class(parent))
        self.apply.setEnabled(False)
        layout.addWidget(self.apply,8,1,1,1)
        self.saveasdefault = QtGui.QPushButton('save as default')
        self.saveasdefault.clicked.connect(lambda: self.save_default(parent))
        self.saveasdefault.setEnabled(False)
        layout.addWidget(self.saveasdefault,8,2,1,1)
        done = QtGui.QPushButton('close')
        done.clicked.connect(self.exit_list)
        layout.addWidget(done,8,3,1,1)

    def load_cell(self):
        name = QtGui.QFileDialog.getOpenFileName(self, 'Open iscell.npy file',filter='iscell.npy')
        if name:
            try:
                iscell = np.load(name[0])
                badfile = True
                if iscell.shape[0] > 0:
                    if iscell[0,0]==0 or iscell[0,0]==1:
                        badfile = False
                        self.list.addItem(name[0])
                if badfile:
                    QtGui.QMessageBox.information(self, 'iscell.npy should be 0/1')
            except (OSError, RuntimeError, TypeError, NameError):
                QtGui.QMessageBox.information(self, 'iscell.npy should be 0/1')
        else:
            QtGui.QMessageBox.information(self, 'iscell.npy should be 0/1')

    def load_text(self):
        name = QtGui.QFileDialog.getOpenFileName(self, 'Open *.txt file', filter='text file (*.txt)')
        if name:
            try:
                txtfile = open(name[0], 'r')
                files = txtfile.read()
                txtfile.close()
                files = files.splitlines()
                for f in files:
                    self.list.addItem(f)
            except (OSError, RuntimeError, TypeError, NameError):
                QtGui.QMessageBox.information(self, 'not a text file')
                print('not a good list')

    def build_classifier(self, parent):
        parent.trainfiles = []
        i=0
        for item in self.list.selectedItems():
            parent.trainfiles.append(item.text())
            i+=1
        if i==0:
            for r in range(self.list.count()):
                parent.trainfiles.append(self.list.item(r).text())
        if len(parent.trainfiles)>0:
            print('Populating classifier:')
            keys = parent.default_keys
            loaded = load_data(parent, keys, parent.trainfiles)
            if loaded:
                msg = QtGui.QMessageBox.information(parent,'Classifier saved',
                                                    'Classifier built from valid files and saved.')
                self.apply.setEnabled(True)
                self.saveasdefault.setEnabled(True)

    def apply_class(self, parent):
        parent.model = Classifier(classfile=parent.classfile)
        activate(parent, True)

    def save_default(self, parent):
        dm = QtGui.QMessageBox.question(self,'Default classifier',
                                        'Are you sure you want to overwrite your default classifier?',
                                        QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        if dm == QtGui.QMessageBox.Yes:
            classorig = parent.classfile
            classfile = os.path.join(os.path.abspath(os.path.dirname(__file__)),
                             'classifiers/classifier_user.npy')
            shutil.copy(classorig, classfile)

    def exit_list(self):
        self.accept()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="suite2p",
    version="0.2.8",
    author="Marius Pachitariu and Carsen Stringer",
    author_email="marius10p@gmail.com",
    description="Pipeline for calcium imaging",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MouseLand/suite2p",
    packages=setuptools.find_packages(),
	install_requires = ['pyqtgraph', 'PyQt5', 'numpy', 'scipy', 'h5py', 'scikit-image', 'matplotlib'],
    include_package_data=True,
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ),
)



import numpy as np


def binMovie(ops):
    mov = np.zeros((ops['Ly'], ops['Lx'], ops['navg_frames_svd']), np.float32)

    with open(ops['reg_file'], 'r') as binary_file:
    # Read the whole file at once
    data = binary_file.read()
    print(data)
    return mov


### custom QPushButton class that plots image when clicked
# requires buttons to put into a QButtonGroup (parent.viewbtns)
# allows up to 1 button to pressed at a time
class ViewButton(QtGui.QPushButton):
    def __init__(self, bid, Text, parent=None):
        super(ViewButton,self).__init__(parent)
        self.setText(Text)
        self.setCheckable(True)
        self.resize(self.minimumSizeHint())
        self.clicked.connect(lambda: self.press(parent, bid))
        self.show()
    def press(self, parent, bid):
        ischecked  = parent.viewbtns.checkedId()
        waschecked = parent.btnstate[bid]
        for n in range(len(parent.btnstate)):
            parent.btnstate[n] = False
        if ischecked==bid and not waschecked:
            parent.viewbtns.setExclusive(True)
            parent.ops_plot[1] = bid+1
            M = fig.draw_masks(parent)
            parent.plot_masks(M)
            parent.btnstate[bid]=True
        elif ischecked==bid and waschecked:
            parent.viewbtns.setExclusive(False)
            parent.btnstate[bid]=False
            parent.ops_plot[1] = 0
            M = fig.draw_masks(parent)
            parent.plot_masks(M)
        self.setChecked(parent.btnstate[bid])
        
### custom QDialog which makes a list of items you can include/exclude
class ListChooser(QtGui.QDialog):
    def __init__(self, Text, parent=None):
        super(ListChooser, self).__init__(parent)
        self.setGeometry(300,300,650,320)
        self.setWindowTitle(Text)
        self.win = QtGui.QWidget(self)
        layout = QtGui.QGridLayout()
        self.win.setLayout(layout)
        #self.setCentralWidget(self.win)
        loadtext = QtGui.QPushButton('Load txt file')
        loadtext.resize(loadtext.minimumSizeHint())
        loadtext.clicked.connect(self.load_text)
        layout.addWidget(loadtext,0,0,1,1)
        self.leftlist = QtGui.QListWidget(parent)
        self.rightlist = QtGui.QListWidget(parent)
        layout.addWidget(QtGui.QLabel('INCLUDE'),1,0,1,1)
        layout.addWidget(QtGui.QLabel('EXCLUDE'),1,3,1,1)
        layout.addWidget(self.leftlist,2,0,5,1)
        #self.leftlist.setSelectionMode(QtGui.QAbstractItemView.MultiSelection)
        sright = QtGui.QPushButton('-->')
        sright.resize(sright.minimumSizeHint())
        sleft = QtGui.QPushButton('<--')
        sleft.resize(sleft.minimumSizeHint())
        sright.clicked.connect(self.move_right)
        sleft.clicked.connect(self.move_left)
        layout.addWidget(sright,3,1,1,1)
        layout.addWidget(sleft,4,1,1,1)
        layout.addWidget(self.rightlist,2,3,5,1)
        done = QtGui.QPushButton('OK')
        done.resize(done.minimumSizeHint())
        done.clicked.connect(lambda: self.exit_list(parent))
        layout.addWidget(done,7,1,1,1)
    def move_right(self):
        currentRow = self.leftlist.currentRow()
        if self.leftlist.item(currentRow) is not None:
            self.rightlist.addItem(self.leftlist.item(currentRow).text())
            self.leftlist.takeItem(currentRow)
    def move_left(self):
        currentRow = self.rightlist.currentRow()
        if self.rightlist.item(currentRow) is not None:
            self.leftlist.addItem(self.rightlist.item(currentRow).text())
            self.rightlist.takeItem(currentRow)
    def load_text(self):
        name = QtGui.QFileDialog.getOpenFileName(self, 'Open File')
        if name:
            try:
                txtfile = open(name[0], 'r')
                files = txtfile.read()
                txtfile.close()
                files = files.splitlines()
                for f in files:
                    self.leftlist.addItem(f)
            except (OSError, RuntimeError, TypeError, NameError):
                print('not a good list')
    def exit_list(self, parent):
        parent.trainfiles = []
        for n in range(len(self.leftlist)):
            if self.leftlist.item(n) is not None:
                parent.trainfiles.append(self.leftlist.item(n).text())
        self.accept()
